@@ -1,6 +1,6 @@
 ;; mew-addrbook.el --- Aliases and personal information
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Mar 22, 1999
 
 ;;; Code:
@@ -537,15 +537,13 @@ reflected."
       (setq file (expand-file-name file mew-conf-path)))
     (switch-to-buffer (mew-find-file-noselect file))
     (emacs-lisp-mode)
-    (if (boundp 'write-file-functions)
-	(add-hook 'write-file-functions 'mew-summary-alias-read-buffer nil 'local)
-      (add-hook 'local-write-file-hooks 'mew-summary-alias-read-buffer))))
+    (add-hook 'write-file-functions 'mew-summary-alias-read-buffer nil 'local)))
 
 (provide 'mew-addrbook)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2015 Mew developing team.
+;; Copyright (C) 1999-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
