@@ -1,6 +1,6 @@
 ;;; mew-attach.el --- attachments for Mew Draft mode
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Oct  2, 1996
 
 ;;; Code:
@@ -827,7 +827,7 @@ Input decrypters' addresses."
        (t
 	(message "Nothing happened"))))))
 
-(defun mew-draft-dnd-handle-remote-url (uri _action)
+(defun mew-draft-dnd-handle-remote-url (_uri _action)
   (error "Remote files not supported") nil)
 
 (defvar mew-draft-dnd-handle-remote-file-function
@@ -847,7 +847,7 @@ Input decrypters' addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2015 Mew developing team.
+;; Copyright (C) 1996-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
