@@ -1,6 +1,6 @@
 ;;; mew-complete.el --- Completion magic for Mew
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: May 30, 1997
 
 ;;; Code:
@@ -14,7 +14,7 @@
 
 (defun mew-draft-on-field-p ()
   (if (bolp)
-      (if (bobp) 
+      (if (bobp)
 	  t
 	(save-excursion
 	  (forward-line -1)
@@ -25,7 +25,7 @@
 	(if (looking-at mew-lwsp)
 	    nil
 	  (if (search-forward ":" pos t) nil t))))))
-      
+
 (defun mew-draft-on-value-p (switch)
   (save-excursion
     (beginning-of-line)
@@ -34,16 +34,16 @@
     (if (looking-at "\\([^:]*:\\)")
 	(mew-field-get-func (match-string 1) switch)
       nil))) ;; what a case reaches here?
-  
+
 ;;
 ;; Window management for completion candidates
 ;;
 
 (defvar mew-complete-candidates nil)
 
-(defun mew-complete-window-delete ()
+(defun mew-complete-window-delete (&optional force)
   (when (mew-ainfo-get-win-cfg)
-    ;; (mew-ainfo-get-win-cfg) remains when the last completion  
+    ;; (mew-ainfo-get-win-cfg) remains when the last completion
     ;; finished with multiple candidates.
     ;; (e.g. foo<RET> when foo and foobar are displayed.)
     ;; In this case, this function is called in another
@@ -52,26 +52,50 @@
     ;; (mew-ainfo-get-win-cfg), the cursor jumps to mini buffer.
     ;; This was a stupid bug of Mew. So, let's see if the complete
     ;; buffer is displayed or not.
-    (if (get-buffer-window mew-buffer-completions)
+    (if (or force (get-buffer-window mew-buffer-completions))
 	(set-window-configuration (mew-ainfo-get-win-cfg)))
     (mew-ainfo-set-win-cfg nil))
   (mew-remove-buffer mew-buffer-completions)
   (setq mew-complete-candidates nil))
 
+(defun mew-complete-insert-folder-function (choice _buffer _mini-p _base-size)
+  (let ((start (mew-minibuf-point-min))
+	(proto (substring choice 0 1))
+	(pos (point)))
+    (while (not (or (= start (point))
+		    (not (char-before))
+		    (char-equal (char-before) ?,)))
+      (forward-char -1))
+    (if (and (member proto mew-folder-prefixes)
+	     (looking-at (concat "\\("
+				 (regexp-opt mew-config-cases t)
+				 ":\\)"
+				 (regexp-quote proto))))
+	(progn
+	  (delete-region (match-end 1) pos)
+	  (goto-char (match-end 1)))
+      (delete-region (point) pos))
+    (insert choice)
+    (remove-text-properties start (point-max) '(mouse-face nil))
+    (mew-complete-window-delete 'force)
+    t))
+
 (defun mew-complete-window-show (all)
   (unless (mew-ainfo-get-win-cfg)
     (mew-ainfo-set-win-cfg (current-window-configuration)))
   (if (and (get-buffer-window mew-buffer-completions)
 	   (equal mew-complete-candidates all))
       (let ((win (get-buffer-window mew-buffer-completions)))
-	(save-excursion
-	  (set-buffer mew-buffer-completions)
+	(with-current-buffer mew-buffer-completions
 	  (if (pos-visible-in-window-p (point-max) win)
 	      (set-window-start win 1)
 	    (scroll-other-window))))
     (setq mew-complete-candidates all)
-    (with-output-to-temp-buffer
-	mew-buffer-completions
+    (with-output-to-temp-buffer mew-buffer-completions
+      (when mew-inherit-complete-folder
+	(make-local-variable 'choose-completion-string-functions)
+	(add-hook 'choose-completion-string-functions
+		  'mew-complete-insert-folder-function))
       (display-completion-list all))))
 
 (defun mew-complete-backscroll ()
@@ -102,7 +126,7 @@ is inserted before the cursor, the short name is expanded to its address."
   (if (mew-draft-on-field-p)
       (mew-complete-field)
     (let ((func (mew-draft-on-value-p mew-field-completion-switch)))
-      (if func 
+      (if func
 	  (funcall func)
 	(tab-to-tab-stop))))) ;; default keybinding
 
@@ -142,7 +166,7 @@ is inserted before the cursor, the short name is expanded to its address."
   (interactive)
   (mew-draft-set-completion-ignore-case mew-complete-address-ignore-case)
   (let ((word (mew-delete-backward-char))
-    	(completion-ignore-case mew-complete-address-ignore-case))
+	(completion-ignore-case mew-complete-address-ignore-case))
     (if (null word)
 	(tab-to-tab-stop)
       (if mew-use-full-alias
@@ -161,7 +185,7 @@ is inserted before the cursor, the short name is expanded to its address."
   (interactive)
   (mew-draft-set-completion-ignore-case mew-complete-address-ignore-case)
   (let ((word (mew-delete-backward-char))
-    	(completion-ignore-case mew-complete-address-ignore-case)
+	(completion-ignore-case mew-complete-address-ignore-case)
 	try)
     (if (null word)
 	(message "No expand key")
@@ -172,20 +196,60 @@ is inserted before the cursor, the short name is expanded to its address."
 	(insert word)
 	(message "'%s' cannot be expanded" word)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Completing folders
+;;;
+
+(defmacro mew-complete-proto-folder (sym &rest body)
+  ;; (declare (indent 1))
+  `(if mew-input-folder-search-direction
+       (mew-input-folder-search-complete)
+     (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
+     (let ((,sym (mew-delete-backward-char))
+	   (completion-ignore-case mew-complete-folder-ignore-case)
+	   (mew-inherit-complete-folder t))
+       ,@body)))
+
+(put 'mew-complete-proto-folder 'lisp-indent-function 1)
+
 (defun mew-complete-local-folder ()
   "Local folder complete function."
   (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-search-complete)
-    (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
-    (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case))
-      (if (null word)
-	  (mew-complete-window-show (list "+"))
-	(if (and (mew-folder-absolutep word)
-		 (not (mew-draft-or-header-p)))
-	    (mew-complete word (mew-complete-directory-alist word) "directory" nil)
-	  (mew-complete word (mew-local-folder-alist) "folder" nil))))))
+  (mew-complete-proto-folder word
+    (if (null word)
+       (mew-complete-window-show (list "+"))
+     (if (and (mew-folder-absolutep word)
+	      (not (mew-draft-or-header-p)))
+	 (mew-complete word (mew-complete-directory-alist word) "directory" nil)
+       (mew-complete word (mew-local-folder-alist) "folder" nil)))))
+
+;; case is specified by mew-inherit-case.
+(defun mew-complete-imap-folder ()
+  "IMAP folder complete function."
+  (interactive)
+  (mew-complete-proto-folder word
+    (if (null word)
+	(mew-complete-window-show (list "%"))
+      (mew-complete
+       word
+       (mew-imap-folder-alist mew-inherit-case) ;; ie mew-sinfo-get-case
+       "mailbox"
+       nil))))
+
+(defun mew-complete-fcc-folder ()
+  "Fcc: folder complete function."
+  (interactive)
+  (mew-complete-proto-folder word
+    (if (null word)
+	(mew-complete-window-show (list "+" "%"))
+      (cond
+       ((and (mew-folder-absolutep word) (not (mew-draft-or-header-p)))
+	(mew-complete word (mew-complete-directory-alist word) "directory" nil))
+       ((mew-folder-imapp word)
+	(mew-complete word (mew-imap-folder-alist (mew-tinfo-get-case)) "mailbox" nil))
+       (t
+	(mew-complete word (mew-local-folder-alist) "folder" nil))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -200,9 +264,9 @@ is inserted before the cursor, the short name is expanded to its address."
     (mew-complete-folder2)))
 
 (defun mew-input-folder-search-complete ()
-  (let (keys)
-    (save-excursion
-      (set-buffer mew-input-folder-search-buf)
+  (let ((mew-inherit-complete-folder t)
+	keys)
+    (with-current-buffer mew-input-folder-search-buf
       (save-excursion
 	(goto-char (point-min))
 	(while (search-forward (or mew-input-folder-search-key "\n") nil t)
@@ -216,6 +280,7 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-complete-folder2 ()
   (let ((word (mew-delete-backward-char nil ", \t\n"))
 	(completion-ignore-case mew-complete-folder-ignore-case)
+	(mew-inherit-complete-folder t)
 	case folder)
     (cond
      ((null word)
@@ -265,28 +330,11 @@ is inserted before the cursor, the short name is expanded to its address."
 	 "case"
 	 nil)))))))
 
-;; called by mew-input-refile-folders.
-;; case is specified by mew-inherit-case.
-(defun mew-complete-imap-folder ()
-  "IMAP folder complete function."
-  (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-search-complete)
-    (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case))
-      (if (null word)
-	  (mew-complete-window-show (list "%"))
-	(mew-complete
-	 word
-	 (mew-imap-folder-alist mew-inherit-case) ;; ie mew-sinfo-get-case
-	 "mailbox"
-	 nil)))))
-
 (defun mew-complete-case ()
   "Complete function for cases."
   (interactive)
   (let ((word (or (mew-delete-backward-char) ""))
-    	(completion-ignore-case mew-complete-case-ignore-case))
+	(completion-ignore-case mew-complete-case-ignore-case))
     (mew-complete
      word
      (mapcar 'list mew-config-cases)
@@ -309,8 +357,8 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-circular-complete-domain ()
   "Circular completion of domains for To:, Cc:, etc.
 If the @ character does not exist, the first value of
-mew-mail-domain-list is inserted. If exists, the next value of 
-mew-mail-domain-list concerned with the string between @ and 
+mew-mail-domain-list is inserted. If exists, the next value of
+mew-mail-domain-list concerned with the string between @ and
 the cursor is inserted."
   (interactive)
   (mew-draft-set-completion-ignore-case
@@ -491,7 +539,7 @@ the name exists."
 	(progn
 	  ;; If WORD contains ':', change alist for completion.
 	  (setq field (car (mew-split word ?:)))
-	  (setq alist 
+	  (setq alist
 		(mapcar (lambda (str) (list (concat field ":" str))) mew-sort-modes)))
       ;; Otherwise, alist is mew-sort-key-alist itself.
       (setq alist mew-sort-key-alist))
@@ -571,7 +619,7 @@ the name exists."
 	  (message
 	   "To expand '%s', type '%c' then '%s'"
 	   cmp EXPAND-CHAR
- 	   (substitute-command-keys
+	   (substitute-command-keys
 	    "\\<mew-draft-header-map>\\[mew-draft-header-comp]"))))
      ;; no candidate
      (t
@@ -628,9 +676,7 @@ the name exists."
       (delete-region savemax (point-max))
       (when quit-flag
 	(setq quit-flag nil)
-	(setq unread-command-events (list (if mew-xemacs-p
-					      (character-to-event 7)
-					    7))))))) ;; 7 == C-g
+	(setq unread-command-events (list 7)))))) ;; 7 == C-g
 
 ;;
 ;; Extracting completion key
@@ -667,7 +713,7 @@ the name exists."
 
 (defun mew-delete-pattern ()
   (let ((pos (point)))
-    (if (re-search-backward " \\|(\\|&\\||\\|!" nil t)
+    (if (re-search-backward " \\|(\\|&\\||\\|!\\|," nil t)
 	(forward-char 1)
       (beginning-of-line))
     (prog1
@@ -721,7 +767,7 @@ the name exists."
   (let ((len (length x))
 	(ret nil)
 	(first (car x)))
-    (cond 
+    (cond
      ((= len 0) nil)
      ((= len 1) (list (cons first first)))
      (t
@@ -735,13 +781,13 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2007 Mew developing team.
+;; Copyright (C) 1997-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -750,7 +796,7 @@ the name exists."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
