@@ -1,23 +1,12 @@
 ;;; mew-env.el --- Environment setup for Mew
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Mar  6, 1997
 
 ;;; Code:
 
 (require 'mew)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Suppressing warnings at compile
-;;;
-
-(defmacro mew-no-warning-defvar (var-name)
-  `(unless (boundp ',var-name) (defvar ,var-name nil)))
-
-(defmacro mew-no-warning-defun (func-name)
-  `(unless (fboundp ',func-name) (defun ,func-name(&rest args) nil)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;;
@@ -70,7 +59,7 @@ requires PTY.")
 
 (if (fboundp 'mouse-region-match)
     (defalias 'mew-mouse-region-p 'mouse-region-match)
-  (defmacro mew-mouse-region-p (&rest args) nil))
+  (defmacro mew-mouse-region-p (&rest _args) nil))
 
 (cond
  ((boundp 'auto-hscroll-mode) ;; Emacs 21.3.50 or later
@@ -86,7 +75,9 @@ requires PTY.")
 
 (eval-when-compile
   (unless (fboundp 'with-no-warnings)
-    (defmacro with-no-warnings (&rest body) `(progn ,@body))))
+    (defmacro with-no-warnings (&rest body)
+      (declare (debug (&rest form)))
+      `(progn ,@body))))
 
 (if (fboundp 'set-process-query-on-exit-flag)
     (defun mew-process-silent-exit (pro)
@@ -97,7 +88,8 @@ requires PTY.")
 
 (with-no-warnings
   (defun mew-set-coding-priority (pri)
-    (set-coding-priority pri)))
+    (apply 'set-coding-system-priority
+	   (mapcar (lambda (x) (symbol-value x)) pri))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -171,7 +163,7 @@ requires PTY.")
 (if (and (fboundp 'set-file-times)
 	 (memq system-type '(darwin windows-nt cygwin)))
     (defalias 'mew-set-file-times 'set-file-times)
-  (defmacro mew-set-file-times (&rest args) nil))
+  (defmacro mew-set-file-times (&rest _args) nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -221,11 +213,11 @@ requires PTY.")
 
 (if (fboundp 'unix-sync)
     (defalias 'mew-unix-sync 'unix-sync)
-  (defmacro mew-unix-sync (&rest args) nil))
+  (defmacro mew-unix-sync (&rest _args) nil))
 
 (if (fboundp 'mac-set-file-type)
     (defalias 'mew-mac-set-file-type 'mac-set-file-type)
-  (defmacro mew-mac-set-file-type (&rest args) nil))
+  (defmacro mew-mac-set-file-type (&rest _args) nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -289,46 +281,33 @@ requires PTY.")
 
 (cond
  ((memq system-type '(windows-nt cygwin))
-  (defun mew-set-file-type (file) ())
+  (defun mew-set-file-type (_file) ())
   (defvar mew-cs-est 'shift_jis)
   (defun mew-focus-frame (frame)
     (if (fboundp 'w32-focus-frame) (w32-focus-frame frame))))
  ((eq system-type 'darwin)
-  (defun mew-set-file-type (file)
+  (defun mew-set-file-type (_file) ;; prevent byte-compile warning
     (unless mew-use-suffix
-      (mew-mac-set-file-type file mew-file-type)))
+      (mew-mac-set-file-type _file mew-file-type)))
   (defvar mew-cs-est 'utf-8)
-  (defun mew-focus-frame (frame)
+  (defun mew-focus-frame (_frame)
     (when focus-follows-mouse
       (set-mouse-position
        (selected-frame) (1- (frame-width)) 0))))
  (t
-  (defun mew-set-file-type (file) ())
+  (defun mew-set-file-type (_file) ())
   (defvar mew-cs-est 'utf-8)
-  (defun mew-focus-frame (frame)
+  (defun mew-focus-frame (_frame)
     (when focus-follows-mouse
       (set-mouse-position
        (selected-frame) (1- (frame-width)) 0)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Unix/Mac/Win
-;;;
-
-(cond 
- ((>= emacs-major-version 24)
-  ;; this must be macro. If implemented as a function, its behavior
-  ;; is changed.
-  (defmacro mew-called-interactively-p ()
-    '(called-interactively-p 'interactive)))
- (t
-  (defalias 'mew-called-interactively-p 'called-interactively-p)))
 
 (provide 'mew-env)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2015 Mew developing team.
+;; Copyright (C) 1997-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
