@@ -1,6 +1,6 @@
 ;;; mew-imap.el for reading
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Feb 21, 2002
 
 ;;; Code:
@@ -1449,7 +1449,7 @@
 			(mew-imap-secure-p pnm))
        (goto-char (point-min))
        (when (and (null bytes)
-		  (looking-at "^\\*[^{]*{\\([0-9]+\\)}"))
+		  (looking-at "^\\*[^{]*{\\([0-9]+\\)}.*\n"))
 	 (setq bytes (string-to-number (mew-match-string 1)))
 	 (mew-imap-set-bytes pnm bytes))
        (when (looking-at "^\\* .*X-GM-THRID +\\([0-9]+\\)")
@@ -1888,7 +1888,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2015 Mew developing team.
+;; Copyright (C) 2002-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
