@@ -1,6 +1,6 @@
 ;;; mew-mark.el --- Marking for Mew Summary and Virtual mode
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Mar  2, 1997
 
 ;;; Code:
@@ -24,7 +24,7 @@
 ;;; Undo
 ;;;
 
-(defsubst mew-mark-unmark ()
+(defun mew-mark-unmark ()
   (save-excursion
     (mew-summary-goto-mark)
     (when (looking-at mew-regex-mark)
@@ -51,7 +51,7 @@
 	  (setq msg (mew-summary-message-number))
 	  (mew-mark-unmark)
 	  (if func (funcall func fld msg))
-	  (if (mew-virtual-p)
+	  (if (mew-virtual-for-one-summary)
 	      (mew-summary-unmark-in-physical fld msg func)))))))
 
 ;;
@@ -71,6 +71,7 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 	 (func (mew-markdb-func-undo mark))
 	 (case-fold-search nil)
 	 (reviewp (char-equal mark mew-mark-review))
+	 (one-summary (and (not virtual-only) (mew-virtual-for-one-summary)))
 	 alist fld msg reviews)
     (or (fboundp func) (setq func nil))
     (save-excursion
@@ -79,15 +80,13 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
 	(mew-mark-remove)
-	(when (mew-virtual-p)
-	  (mew-mark-alist-set alist fld msg))
+	(if one-summary (mew-mark-alist-set alist fld msg))
 	(when reviewp (setq reviews (cons msg reviews)))
 	(if func (funcall func fld msg))
 	(forward-line))
       (set-buffer-modified-p nil)
       (mew-sinfo-set-mark-review (nreverse reviews))
-      (unless virtual-only
-	(mew-summary-unmark-in-physical-alist alist func))))
+      (if one-summary (mew-summary-unmark-in-physical-alist alist func))))
   (or no-msg (message "Unmarking...done")))
 
 ;;
@@ -95,12 +94,10 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 (defun mew-summary-mark-undo-all ()
   "Unmark all message marked with 'o' or 'D' or 'X'."
   (interactive)
-  (let ((marks mew-summary-mark-undo-marks))
-    (message "Unmarking...")
-    (while marks
-      (mew-mark-undo-mark (car marks) 'no-msg)
-      (setq marks (cdr marks)))
-    (message "Unmarking...done")))
+  (message "Unmarking...")
+  (dolist (mark mew-summary-mark-undo-marks)
+    (mew-mark-undo-mark mark 'no-msg))
+  (message "Unmarking...done"))
 
 ;;
 
@@ -108,14 +105,11 @@ If optional argument NO-MSG is non-nil, no message is displayed."
   "\\<mew-summary-mode-map>Recover the `*' marks which are canceled by the last `\\[mew-summary-undo-all]*'."
   (interactive)
   (let ((mark mew-mark-review)
-	(reviews (mew-sinfo-get-mark-review))
-	msg)
+	(reviews (mew-sinfo-get-mark-review)))
     (save-excursion
       (mew-mark-undo-mark mark 'no-msg)
       (goto-char (point-min))
-      (while reviews
-	(setq msg (car reviews))
-	(setq reviews (cdr reviews))
+      (dolist (msg reviews)
 	(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 	  (mew-mark-put mark)
 	  (forward-line))))
@@ -143,12 +137,11 @@ If optional argument NO-MSG is non-nil, no message is displayed."
   (when (mew-summary-markable)
     (mew-mark-put mark)))
 
-(defsubst mew-mark-put (mark)
+(defun mew-mark-put (mark)
   (save-excursion
     (mew-summary-goto-mark)
     (let* ((beg (point)) (end (1+ beg)))
       (mew-elet
-       (mew-front-sticky beg end) ;; for XEmacs
        ;; This code is awkward but for invisible.
        (forward-char 1)
        (insert-and-inherit (char-to-string mark)) ;; inherit highlight
@@ -158,13 +151,12 @@ If optional argument NO-MSG is non-nil, no message is displayed."
        (mew-highlight-mark-line mark))
       (mew-mark-hist-set (mew-summary-message-number) mark))))
 
-(defsubst mew-mark-remove ()
+(defun mew-mark-remove ()
   (save-excursion
     (mew-summary-goto-mark)
     (let* ((mark mew-mark-read)
 	   (beg (point)) (end (1+ beg)))
       (mew-elet
-       (mew-front-sticky beg end) ;; for XEmacs
        ;; This code is awkward but for invisible.
        (forward-char 1)
        (insert-and-inherit (char-to-string mark)) ;; inherit highlight
@@ -198,7 +190,7 @@ If optional argument NO-MSG is non-nil, no message is displayed."
       (nreverse msglist))))
 
 (defun mew-summary-mark-collect2 (mark)
-  "For virtual mode, this function returns a list of
+  "For Virtual mode, this function returns a list of
 cons pairs of folder name and message number."
   (save-excursion
     (let ((regex (mew-mark-regex mark))
@@ -206,7 +198,7 @@ cons pairs of folder name and message number."
 	  (case-fold-search nil))
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
-        (setq msglist (cons 
+        (setq msglist (cons
                        (cons
                         (mew-summary-folder-name)
                         (mew-summary-message-number))
@@ -241,11 +233,9 @@ cons pairs of folder name and message number."
 (defun mew-summary-mark-recover (mdb &optional refdb refs)
   (let ((opos (point))
 	(case-fold-search nil)
-	ent msg mrk new-ref ref-ent)
+	msg mrk new-ref ref-ent)
     (goto-char (point-min))
-    (while mdb
-      (setq ent (car mdb))
-      (setq mdb (cdr mdb))
+    (dolist (ent mdb)
       (mew-set '(msg mrk) ent)
       (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 	(if (or (not (char-equal mrk mew-mark-refile)) (null refdb))
@@ -254,10 +244,9 @@ cons pairs of folder name and message number."
 	    (mew-mark-put mrk)
 	    (setq new-ref (cons ref-ent new-ref))))
 	(forward-line)))
-    (while refs
-      (when (setq ref-ent (assoc (car refs) refdb))
-	(setq new-ref (cons ref-ent new-ref)))
-      (setq refs (cdr refs)))
+    (dolist (ref refs)
+      (when (setq ref-ent (assoc ref refdb))
+	(setq new-ref (cons ref-ent new-ref))))
     (goto-char opos)
     (nreverse new-ref)))
 
@@ -293,6 +282,8 @@ See also mew-mark-afterstep-spec."
       (let ((mew-inherit-after-marking t))
 	(mew-summary-display-after mew-summary-mark-direction))))))
 
+(mew-defstruct mark-hist msg mark)
+
 (defun mew-mark-hist-set (msg mark)
   (let* ((hist (mew-sinfo-get-mark-hist))
 	 (ent (assoc msg hist))
@@ -302,7 +293,7 @@ See also mew-mark-afterstep-spec."
      (ent
       (if mark (setcar (nthcdr 1 ent) mark)))
      (t
-      (setq ent (list msg mark))
+      (setq ent (mew-make-mark-hist :msg msg :mark mark))
       (setq curr hist)
       (catch 'loop
 	(while curr
@@ -373,19 +364,19 @@ NO-MSG also means that this function is being called in loop."
 	   (message "Cannot mark here because '%s' is stronger than '%s'"
 		    oldname newname)))))
      (when validp
-       (if (and marked (mew-virtual-p))
+       (if (and marked (mew-virtual-for-one-summary))
 	   (mew-summary-mark-in-physical fld msg newmark))
        (or no-msg (mew-mark-afterstep newmark case))
        (set-buffer-modified-p nil)))))
 
 (defun mew-mark-put-mark-loop (func count stayp)
-  "Unless COUNT is numeric, just call FUNC once. 
+  "Unless COUNT is numeric, just call FUNC once.
 The cursor moves forward. STAYP has no effect.
 If COUNT is positive, call FUNC in COUNT times moving the cursor forward.
 If COUNT is negative, call FUNC in COUNT times moving the cursor backward.
 If COUNT is numeric and STAYP is non-nil, the cursor stays in the
 original position."
-  (if (and (not (numberp count)) (mew-mark-active-p))
+  (if (and (not (integerp count)) (mew-mark-active-p))
       (let ((begend (mew-summary-get-region)))
 	(goto-char (car begend))
 	(setq stayp nil)
@@ -396,7 +387,7 @@ original position."
 (defun mew-mark-put-mark-loop1 (func count stayp)
   (when (and func (fboundp func))
     (mew-summary-msg-or-part
-     (if (numberp count)
+     (if (integerp count)
 	 (let ((start (point)))
 	   (mew-decode-syntax-delete)
 	   ;; positive loop
@@ -450,19 +441,16 @@ original position."
 
 (defun mew-summary-mark-in-physical-alist (alist mark &optional func)
   (save-excursion
-    (let (ent fld msg msgs)
-      (while alist
-	(setq ent (car alist))
-	(setq alist (cdr alist))
+    (let (fld msg msgs)
+      (dolist (ent alist)
 	(setq fld (car ent))
 	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
 	(when (get-buffer fld)
 	  (set-buffer fld)
 	  (save-excursion
 	    (goto-char (point-min))
-	    (while msgs
-	      (setq msg (number-to-string (car msgs)))
-	      (setq msgs (cdr msgs))
+	    (dolist (m msgs)
+	      (setq msg (number-to-string m))
 	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 		(if func (funcall func))
 		(mew-mark-put mark)
@@ -471,19 +459,16 @@ original position."
 
 (defun mew-summary-unmark-in-physical-alist (alist func)
   (save-excursion
-    (let (ent fld msg msgs)
-      (while alist
-	(setq ent (car alist))
-	(setq alist (cdr alist))
+    (let (fld msg msgs)
+      (dolist (ent alist)
 	(setq fld (car ent))
 	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
 	(when (get-buffer fld)
 	  (set-buffer fld)
 	  (save-excursion
 	    (goto-char (point-min))
-	    (while msgs
-	      (setq msg (number-to-string (car msgs)))
-	      (setq msgs (cdr msgs))
+	    (dolist (m msgs)
+	      (setq msg (number-to-string m))
 	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 		(mew-mark-remove)
 		(if func (funcall func fld msg))
@@ -588,18 +573,19 @@ BEG and END."
   (interactive "r")
   (let ((regex (mew-mark-regex mew-sp)) ;; not marked
 	(mark mew-mark-review) ;; someday ...
+	(one-summary (mew-virtual-for-one-summary))
 	fld msg alist)
     (save-excursion
       (goto-char beg)
       (while (re-search-forward regex end t)
 	(mew-summary-mark-as mark)
-	(when (mew-virtual-p)
+	(when one-summary
 	  (setq fld (mew-summary-folder-name))
 	  (setq msg (mew-summary-message-number))
 	  (mew-mark-alist-set alist fld msg))
 	(forward-line))
       (set-buffer-modified-p nil))
-    (mew-summary-mark-in-physical-alist alist mark)))
+    (if one-summary (mew-summary-mark-in-physical-alist alist mark))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -613,13 +599,14 @@ BEG and END."
       (unless (search-forward "\r" (match-end 0) t)
 	(throw 'loop t))
       (forward-line))))
-    
+
 (defun mew-summary-mark-regexp (&optional args)
   "Put the '*' mark onto all messages matched to a regular expression."
   (interactive "P")
   (mew-decode-syntax-delete)
   (let ((regex (read-string "Regexp: "))
 	(mark mew-mark-review) ;; someday ...
+	(one-summary (mew-virtual-for-one-summary))
         (n 0)
 	fld msg alist)
     (while (string= regex "")
@@ -631,7 +618,7 @@ BEG and END."
 	       (mew-summary-search-regexp-visible regex))
 	(when (and (mew-summary-markable)
 		   (mew-sumsyn-match mew-regex-sumsyn-short))
-	  (when (mew-virtual-p)
+	  (when one-summary
 	    (setq fld (mew-sumsyn-folder-name))
 	    (setq msg (mew-sumsyn-message-number))
 	    (mew-mark-alist-set alist fld msg))
@@ -639,7 +626,7 @@ BEG and END."
 	  (setq n (1+ n)))
 	(forward-line))
       (set-buffer-modified-p nil))
-    (mew-summary-mark-in-physical-alist alist mark)
+    (if one-summary (mew-summary-mark-in-physical-alist alist mark))
     (cond
      ((= n 1)
       (message "1 message marked"))
@@ -656,7 +643,9 @@ BEG and END."
 (defun mew-summary-exchange-mark (oldmark newmark &optional valid-only)
   (let ((regex (mew-mark-regex oldmark))
 	(case-fold-search nil)
-	(i 0) fld msg alist)
+	(one-summary (mew-virtual-for-one-summary))
+	(i 0)
+	fld msg alist)
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
@@ -665,15 +654,14 @@ BEG and END."
 	  (setq msg (mew-sumsyn-message-number))
 	  (when (or (not valid-only) (mew-msg-validp msg))
 	    (mew-mark-put newmark)
-	    (when (mew-virtual-p)
-	      (mew-mark-alist-set alist fld msg))))
+	    (if one-summary (mew-mark-alist-set alist fld msg))))
 	(forward-line)
 	(setq i (1+ i))))
     (if (= i 0)
 	(message "No marked messages")
       (set-buffer-modified-p nil)
-      (mew-summary-mark-in-physical-alist alist newmark))))
-   
+      (if one-summary (mew-summary-mark-in-physical-alist alist newmark)))))
+
 (defun mew-summary-mark-delete ()	;; * -> D
   "Put the delete mark onto all messages marked with '*'."
   (interactive)
@@ -813,11 +801,11 @@ region."
 (defun mew-summary-down-mark (mark)
   (let ((case-fold-search nil))
     (forward-line)
-    (cond 
+    (cond
      ((re-search-forward (mew-mark-regex mark) nil t)
       (beginning-of-line)
       t)
-     (t 
+     (t
       (forward-line -1)
       (message "No more marked messages")
       nil))))
@@ -825,7 +813,7 @@ region."
 (defun mew-summary-display-review-down (&optional arg)
   "Jump to the message marked with '*' below.
 If called with '\\[universal-argument]', you can change the target mark.
-After that, this command jumps to the message marked with 
+After that, this command jumps to the message marked with
 the specified mark."
   (interactive "P")
   (if arg
@@ -838,14 +826,14 @@ the specified mark."
     (cond
      ((re-search-backward (mew-mark-regex mark) nil t)
       t)
-     (t 
+     (t
       (message "No more marked messages")
       nil))))
 
 (defun mew-summary-display-review-up (&optional arg)
   "Jump to the message marked with '*' above.
 If called with '\\[universal-argument]', you can change the target mark.
-After that, this command jumps to the message marked with 
+After that, this command jumps to the message marked with
 the specified mark."
   (interactive "P")
   (if arg
@@ -901,27 +889,24 @@ The 'o' mark is not exchangeable."
   "Saving marks. Typically called by kill-emacs-hook."
   (remove-hook 'kill-emacs-hook 'mew-sinfo-clean-up)
   (mew-decode-syntax-delete)
-  (let ((bufs mew-buffers) buf)
-    (save-excursion
-      (while bufs
-	(setq buf (car bufs))
-	(setq bufs (cdr bufs))
-	(when (bufferp (get-buffer buf))
-	  (set-buffer buf)
-	  (when (mew-summary-p)
-	    (mew-sinfo-save)))))))
+  (save-excursion
+    (dolist (buf mew-buffers)
+      (when (bufferp (get-buffer buf))
+	(set-buffer buf)
+	(when (mew-summary-p)
+	  (mew-sinfo-save))))))
 
 (provide 'mew-mark)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2007 Mew developing team.
+;; Copyright (C) 1997-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -930,7 +915,7 @@ The 'o' mark is not exchangeable."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
