@@ -1,6 +1,6 @@
 ;;; mew-net.el
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Feb 27, 2002
 
 ;;; Code:
@@ -30,6 +30,7 @@
    ((integerp serv) serv)
    ((string-match "^[0-9]+$" serv)
     (string-to-number serv))
+   ((string-match "^/" serv) serv)
    (t
     (or (mew-alist-get-value (assoc serv mew-port-db)) 0))))
 
@@ -880,7 +881,7 @@ The messages in the server side is always retained."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2015 Mew developing team.
+;; Copyright (C) 2002-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
