@@ -1,6 +1,6 @@
 ;;; mew-nntp2.el for posting
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Nov 19, 1999
 
 ;;; Code:
@@ -334,7 +334,9 @@
 	  (set-buffer buf)
 	  (mew-nntp2-queue case error))
 	(mew-nntp2-log pnm error)
-	(message-box (format "%s  This mail has been queued to %s" error qfld)))
+	(if (memq system-type '(windows-nt ms-dos cygwin))
+	    (message (format "%s  This mail has been queued to %s" error qfld))
+	  (message-box (format "%s  This mail has been queued to %s" error qfld))))
        (done
 	(message "Posting in background...done"))
        (t
@@ -415,7 +417,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2015 Mew developing team.
+;; Copyright (C) 1999-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
