@@ -1,6 +1,6 @@
 ;;; mew-pop.el
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Jun 28, 2000
 
 ;;; Code:
@@ -29,7 +29,7 @@
     "size" "truncated" "get-body"
     "flush" "no-msg" "msgdb" "done" "dispatched" "error"
     "body-lines" "delete" "case"
-    "thread-info" "disp-info" "status-buf"))
+    "virtual-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-pop-" mew-pop-info-list)
 
@@ -55,15 +55,25 @@
     ("dele"          nil ("\\+OK" . "retr"))
     ("quit"          nil ("\\+OK" . "noop"))))
 
-(defsubst mew-pop-fsm-by-status (status)
+(defun mew-pop-fsm-by-status (status)
   (assoc status mew-pop-fsm))
 
-(defsubst mew-pop-fsm-next (status code)
+(defun mew-pop-fsm-next (status code)
   (cdr (mew-assoc-match code (nthcdr 2 (mew-pop-fsm-by-status status)) 0)))
 
-(defsubst mew-pop-fsm-reply (status)
+(defun mew-pop-fsm-reply (status)
   (nth 1 (mew-pop-fsm-by-status status)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Structure
+;;;
+
+(mew-defstruct popinfo num uid size delete fldmsg)
+
+(defun mew-pop-refileinfo-to-popinfo (refileinfo num)
+  (cons num refileinfo))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Filters
@@ -77,7 +87,7 @@
 		  "Auth'ing"
 		  nil
 		  (mew-pop-secure-p pnm))
-  (if (re-search-forward "<[^<>]+@[^<>]+>" nil t)
+  (if (re-search-forward "<[=!-;?-~]+@[=!-;?-~]+>" nil t)
       (mew-pop-set-key pnm (mew-match-string 0)))
   (mew-pop-process-send-string pro "CAPA"))
 
@@ -193,7 +203,7 @@
     (while (re-search-forward "^\\([0-9]+\\) +\\([0-9]+\\)" nil t)
       (setq num (mew-match-string 1))
       (setq siz (mew-match-string 2))
-      (setq msgs (cons (list num nil siz nil) msgs)))
+      (setq msgs (cons (mew-make-popinfo :num num :size siz) msgs)))
     (if msgs
 	(progn
 	  (setq msgs (nreverse msgs))
@@ -205,8 +215,7 @@
 
 ;; The POP server does not support UIDL.
 (defun mew-pop-command-nouidl (pro pnm)
-  (let ((directive (mew-pop-get-directive pnm))
-	rtrs)
+  (let ((directive (mew-pop-get-directive pnm)))
     (if (or (eq directive 'scan) (eq directive 'exec) (eq directive 'sync))
 	(progn
 	  (mew-pop-message pnm "POP server does not support this command")
@@ -214,18 +223,14 @@
       (mew-pop-set-size pnm 0) ;; To get the entire message
       (mew-pop-set-rttl pnm (mew-pop-get-left pnm))
       (when (mew-pop-get-delete pnm)
-	(setq rtrs (mew-pop-get-rtrs pnm)) ;; (num nil siz nil)
-	(while rtrs
-	  (setcar (nthcdr 3 (car rtrs)) t)
-	  (setq rtrs (cdr rtrs))))
+	(dolist (rtr (mew-pop-get-rtrs pnm))
+	  (mew-popinfo-set-delete rtr t)))
       (mew-pop-command-pre-retr pro pnm))))
 
 (defun mew-pop-command-umsg (pro pnm)
   (let* ((directive (mew-pop-get-directive pnm))
-	 (list-msgs (mew-pop-get-rtrs pnm))
-	 ;; (num nil siz nil)            => (num uid siz del)
-	 (refs (mew-pop-get-refs pnm))
-	 ;;     (uid siz del (+fld msg)) => (num uid siz del (+fld msg))
+	 (popinfos (mew-pop-get-rtrs pnm)) ;; popinfo
+	 (refs (mew-pop-get-refs pnm)) ;; refileinfo
 	 (rmvs (mew-pop-get-rmvs pnm))
 	 (del-time (mew-pop-get-delete pnm))
 	 (bnm (mew-pop-get-bnm pnm))
@@ -253,7 +258,7 @@
        ((eq directive 'get)
 	(cond
 	 ((setq rtr (assoc uid refs))
-	  (setq rtr (cons num rtr))
+	  (setq rtr (mew-pop-refileinfo-to-popinfo rtr num))
 	  (setq rtrs (cons rtr rtrs)))
 	 ((member uid rmvs)
 	  (setq dels (cons num dels)))))
@@ -262,15 +267,15 @@
 	    (setq dels (cons num dels))))
        ((eq directive 'biff)
 	(when (and uid (not (assoc uid old-uidl)))
-	  (setq rtr (list num))
+	  (setq rtr (mew-make-popinfo :num num))
 	  (setq rtrs (cons rtr rtrs))))
        ((eq directive 'scan)
 	(when uid
 	  (setq uidl (cons uid uidl))
 	  (when (or range ;; all, last:n
 		    (not (member uid old-uidl))) ;; update
-	    (setq rtr (assoc num list-msgs))
-	    (setcar (nthcdr 1 rtr) uid)
+	    (setq rtr (assoc num popinfos))
+	    (mew-popinfo-set-uid rtr uid)
 	    (setq rtrs (cons rtr rtrs)))))
        ((eq directive 'sync)
 	(when uid
@@ -284,16 +289,14 @@
 	      (setq dels (cons num dels))))
 	 (t
 	  (setq uidl (cons (cons uid ctime) uidl))
-	  (setq rtr (assoc num list-msgs))
-	  (setcar (nthcdr 1 rtr) uid)
-	  (if (eq del-time t)
-	      (setcar (nthcdr 3 rtr) t))
+	  (setq rtr (assoc num popinfos))
+	  (mew-popinfo-set-uid rtr uid)
+	  (if (eq del-time t) (mew-popinfo-set-delete rtr t))
 	  (setq rtrs (cons rtr rtrs)))))))
     (mew-pop-set-uidl pnm uidl)
     ;; last:n
-    (when (and (eq directive 'scan) (numberp range))
-      (if (> (length rtrs) range)
-	  (setcdr (nthcdr (1- range) rtrs) nil)))
+    (when (and (eq directive 'scan) (integerp range))
+      (mew-ntake range rtrs))
     (setq rtrs (nreverse rtrs))
     (setq dels (nreverse dels))
     (setq hlds (nreverse hlds))
@@ -373,14 +376,14 @@
   (mew-net-status2 (mew-pop-get-status-buf pnm)
 		   (mew-pop-get-rttl pnm)
 		   (mew-pop-get-rcnt pnm)
-		   (nth 2 (car (mew-pop-get-rtrs pnm)))
+		   (mew-popinfo-get-size (car (mew-pop-get-rtrs pnm)))
 		   'zero
 		   (mew-pop-secure-p pnm))
   (let* ((directive (mew-pop-get-directive pnm))
 	 (rtrs (mew-pop-get-rtrs pnm))
 	 (rtr  (car rtrs))
-	 (num  (nth 0 rtr))
-	 (siz  (nth 2 rtr))
+	 (num  (mew-popinfo-get-num rtr))
+	 (siz  (mew-popinfo-get-size rtr))
 	 (lim (mew-pop-get-size pnm))
 	 (lines (mew-pop-get-body-lines pnm))
 	 (get-body (mew-pop-get-get-body pnm)))
@@ -408,11 +411,11 @@
 	 (left (mew-pop-get-left pnm))
 	 (rtrs (mew-pop-get-rtrs pnm))
 	 (rtr  (car rtrs))
-	 (num  (nth 0 rtr))
-	 (uid  (nth 1 rtr))
-	 (siz  (nth 2 rtr))
-	 (del  (nth 3 rtr))
-	 (fld-msg (nth 4 rtr))
+	 (num  (mew-popinfo-get-num rtr))
+	 (uid  (mew-popinfo-get-uid rtr))
+	 (siz  (mew-popinfo-get-size rtr))
+	 (del  (mew-popinfo-get-delete rtr))
+	 (fld-msg (mew-popinfo-get-fldmsg rtr))
 	 (truncated (mew-pop-get-truncated pnm))
 	 (case (mew-pop-get-case pnm))
 	 fld msg vec file msg-file lmsg mark folder)
@@ -447,9 +450,9 @@
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
-	  (mew-frwlet 
-	   mew-cs-dummy mew-cs-text-for-write
-	   (write-region (point-min) (point-max) file nil 'no-msg))
+	  (let ((write-region-inhibit-fsync mew-use-async-write))
+	    (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	      (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-pop-set-status pnm "quit")
 	 (mew-pop-command-quit pro pnm)
@@ -490,7 +493,7 @@
   (mew-pop-set-status pnm "quit")
   (mew-pop-process-send-string pro "QUIT"))
 
-(defun mew-pop-command-noop (pro pnm)
+(defun mew-pop-command-noop (_pro _pnm)
   ())
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -502,7 +505,7 @@
   '(("CRAM-MD5" mew-pop-command-auth-cram-md5)
     ("PLAIN"    mew-pop-command-auth-plain)))
 
-(defsubst mew-pop-auth-get-func (auth)
+(defun mew-pop-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-pop-auth-alist 0)))
 
 (defun mew-pop-command-auth-cram-md5 (pro pnm)
@@ -539,20 +542,20 @@
 
 (defconst mew-pop-info-prefix "mew-pop-info-")
 
-(defsubst mew-pop-info-name (case)
+(defun mew-pop-info-name (case)
   (let ((server (mew-pop-server case))
-	(port (mew-pop-port case))
+	(port (mew-*-to-string (mew-pop-port case)))
 	(user (mew-pop-user case))
 	(sshsrv (mew-pop-ssh-server case))
 	(name mew-pop-info-prefix))
     (setq name (concat name user "@" server))
-    (unless (string= port mew-pop-port)
+    (unless (mew-port-equal port mew-pop-port)
       (setq name (concat name ":" port)))
     (if sshsrv
-	(concat name "%" sshsrv)
+        (concat name "%" sshsrv)
       name)))
 
-(defsubst mew-pop-buffer-name (pnm)
+(defun mew-pop-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
 
 (defun mew-pop-process-send-string (pro &rest args)
@@ -562,7 +565,7 @@
 	(process-send-string pro (concat str mew-cs-eol))
       (message "POP time out")))) ;; not mew-pop-message
 
-(defsubst mew-pop-message (pnm &rest args)
+(defun mew-pop-message (pnm &rest args)
   (or (mew-pop-get-no-msg pnm) (apply 'message args)))
 
 (defun mew-pop-message2 (pnm msg left)
@@ -608,14 +611,14 @@
 ;;;
 
 (defun mew-pop-open (pnm server port no-msg)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-pop-timeout-time nil 'mew-pop-timeout))
 	  (or no-msg (message "Connecting to the POP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (or no-msg (message "Connecting to the POP server...done")))
       (quit
@@ -639,7 +642,7 @@
   ;; in +inbox
   (let* ((server (mew-pop-server case))
          (user (mew-pop-user case))
-	 (port (mew-pop-port case))
+	 (port (mew-*-to-string (mew-pop-port case)))
 	 (sshsrv (mew-pop-ssh-server case))
 	 (sslp (mew-pop-ssl case))
 	 (sslport (mew-pop-ssl-port case))
@@ -649,7 +652,7 @@
 	 (buf (get-buffer-create (mew-pop-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport tls
-	 thread-info disp-info virtual)
+	 virtual-info disp-info virtual)
     (if (mew-pop-get-process pnm)
 	(message "Another POP process is running. Try later")
       (cond
@@ -661,7 +664,7 @@
 	  (when lport
 	    (setq process (mew-pop-open pnm "localhost" lport no-msg)))))
        (sslp
-	(if (string= port sslport) (setq tls mew-tls-pop))
+	(if (mew-port-equal port sslport) (setq tls mew-tls-pop))
 	(setq sslpro (mew-open-ssl-stream case server sslport tls))
 	(when sslpro
 	  (setq sslname (process-name sslpro))
@@ -677,6 +680,7 @@
 	      (mew-summary-visible-buffer bnm))
 	(mew-summary-lock process "POPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-summary-column (mew-get-summary-column bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-sinfo-set-scan-md5 nil)
@@ -713,15 +717,14 @@
 	 ((eq directive 'get)
 	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-pop-set-refs pnm (nth 0 args))
-	  (setq thread-info (nth 1 args))
-	  (mew-pop-set-thread-info pnm thread-info)
+	  (setq virtual-info (nth 1 args))
+	  (mew-pop-set-virtual-info pnm virtual-info)
 	  (setq disp-info (nth 2 args))
 	  (mew-pop-set-disp-info pnm disp-info)
-	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (setq virtual (mew-net-virtual-info-get-virtual virtual-info))
 	  (when virtual
 	    (mew-pop-set-status-buf pnm virtual)
-	    (save-excursion
-	      (set-buffer virtual)
+	    (with-current-buffer virtual
 	      (mew-summary-lock process "POPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-pop-set-range pnm (nth 0 args))
@@ -749,8 +752,7 @@
 
 (defun mew-pop-debug (label string)
   (when (mew-debug 'net)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
+    (with-current-buffer (get-buffer-create mew-buffer-debug)
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
@@ -769,7 +771,7 @@
        (mew-net-status2 (mew-pop-get-status-buf pnm)
 			(mew-pop-get-rttl pnm)
 			(mew-pop-get-rcnt pnm)
-			(nth 2 (car (mew-pop-get-rtrs pnm)))
+			(mew-popinfo-get-size (car (mew-pop-get-rtrs pnm)))
 			nil
 			(mew-pop-secure-p pnm)))
      (cond
@@ -813,114 +815,117 @@
 	 (hlds (mew-pop-get-hlds pnm))
 	 (uidl (mew-pop-get-uidl pnm))
 	 (done (mew-pop-get-done pnm))
- 	 (error (mew-pop-get-error pnm))
+	 (error (mew-pop-get-error pnm))
 	 (file (mew-expand-file bnm mew-pop-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-pop-get-thread-info pnm))
+	 (virtual-info (mew-pop-get-virtual-info pnm))
 	 (disp-info (mew-pop-get-disp-info pnm)))
-    (mew-pop-debug "POP SENTINEL" event)
-    (set-process-buffer process nil)
-    (set-buffer bnm)
-    (mew-summary-mark-recover mdb)
-    (mew-remove-buffer buf)
-    (if (not done)
-	(let* ((rtrs (mew-pop-get-rtrs pnm))
-	       (lefts (length rtrs))
-	       (uid (nth 1 (car rtrs)))
-	       recovered)
-	  (mew-pop-message pnm "POP connection is lost")
-	  (when (mew-pop-get-dispatched pnm)
-	    (cond
-	     ((eq directive 'scan)
-	      ;; uidl is reversed.
-	      (setq uidl (cdr (member uid uidl)))
-	      (mew-lisp-save file uidl nil 'unlimit)
-	      (setq recovered t))
-	     ((eq directive 'inc)
-	      ;; uidl is reversed.
-	      (setq uid (assoc uid uidl))
-	      (setq uidl (cdr (member uid uidl)))
-	      (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
-	      (setq recovered t)))
-	    (when recovered
-	      (mew-pop-message
-	       pnm
-	       "%d message retrieved. %d messages are left due to an error"
-	       (- rttl lefts) lefts)
-	      (mew-summary-folder-cache-save))))
-      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
-      (cond
-       (error 
- 	;; retain the error message
- 	)
-       ((eq directive 'biff)
-	(funcall mew-biff-function rttl))
-       ((eq directive 'sync)
-	(mew-pop-message pnm "Synchronizing messages...")
-	(mew-net-folder-sync bnm hlds)
-	(mew-pop-message pnm "Synchronizing messages...done"))
-       ((eq directive 'inc)
-	(mew-biff-clear)
-	(mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+    (save-excursion
+      (mew-pop-debug "POP SENTINEL" event)
+      (set-process-buffer process nil)
+      (set-buffer bnm)
+      (mew-summary-mark-recover mdb)
+      (mew-remove-buffer buf)
+      (if (not done)
+	  (let* ((rtrs (mew-pop-get-rtrs pnm))
+		 (lefts (length rtrs))
+		 (uid (mew-popinfo-get-uid (car rtrs)))
+		 recovered)
+	    (mew-pop-message pnm "POP connection is lost")
+	    (when (mew-pop-get-dispatched pnm)
+	      (cond
+	       ((eq directive 'scan)
+		;; uidl is reversed.
+		(setq uidl (cdr (member uid uidl)))
+		(mew-lisp-save file uidl nil 'unlimit)
+		(setq recovered t))
+	       ((eq directive 'inc)
+		;; uidl is reversed.
+		(setq uid (assoc uid uidl))
+		(setq uidl (cdr (member uid uidl)))
+		(mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+		(setq recovered t)))
+	      (when recovered
+		(mew-pop-message
+		 pnm
+		 "%d message retrieved. %d messages are left due to an error"
+		 (- rttl lefts) lefts)
+		(mew-summary-folder-cache-save))))
+	(if virtual-info (mew-summary-retrieve-message-for-virtual virtual-info))
 	(cond
-	 ((= rttl 0)
-	  (mew-pop-message2 pnm "No new messages" left))
-	 ((= rttl 1)
-	  (mew-pop-message2 pnm "1 message retrieved" left)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'get)
-	(cond
-	 ((= rttl 0)
-	  (mew-pop-message2 pnm "No new messages" left))
-	 ((= rttl 1)
-	  (mew-pop-message2 pnm "1 message retrieved" left)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'scan)
-	(mew-biff-clear)
-	(cond
-	 ((= rttl 0)
-	  (mew-pop-message pnm "No messages scanned"))
-	 ((= rttl 1)
-	  (mew-pop-message pnm "1 message scanned")
-	  (mew-lisp-save file uidl nil 'unlimit)
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-pop-message pnm "%d messages scanned" rttl)
-	  (mew-lisp-save file uidl nil 'unlimit)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'exec)
-	(when kils
-	  (mew-mark-exec-unlink bnm kils)
-	  (mew-mark-kill-invisible)
-	  (mew-summary-folder-cache-save))
-	(cond
-	 ((= dttl 0)
-	  (mew-pop-message pnm "No messages deleted"))
-	 ((= dttl 1)
-	  (mew-pop-message pnm "1 message deleted"))
-	 (t
-	  (mew-pop-message pnm "%d messages deleted" dttl))))))
-    ;;
-    (mew-net-status-clear (mew-pop-get-status-buf pnm))
-    (mew-info-clean-up pnm)
-    (set-buffer-modified-p nil)
-    (mew-summary-unlock)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (mew-net-disp-info-display disp-info)
-    (unless (eq directive 'biff)
-      (run-hooks 'mew-pop-sentinel-non-biff-hook))
-    (run-hooks 'mew-pop-sentinel-hook)
-    (when (and mew-auto-flush-queue flush)
-      (mew-smtp-flush-queue mew-case))))
+	 (error
+	  ;; retain the error message
+	  )
+	 ((eq directive 'biff)
+	  (and (functionp mew-biff-function)
+	       (funcall mew-biff-function rttl)))
+	 ((eq directive 'sync)
+	  (mew-pop-message pnm "Synchronizing messages...")
+	  (mew-net-folder-sync bnm hlds)
+	  (mew-pop-message pnm "Synchronizing messages...done"))
+	 ((eq directive 'inc)
+	  (mew-biff-clear)
+	  (mew-net-uidl-db-set (mew-pop-passtag pnm) uidl)
+	  (cond
+	   ((= rttl 0)
+	    (mew-pop-message2 pnm "No new messages" left))
+	   ((= rttl 1)
+	    (mew-pop-message2 pnm "1 message retrieved" left)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'get)
+	  (cond
+	   ((= rttl 0)
+	    (mew-pop-message2 pnm "No new messages" left))
+	   ((= rttl 1)
+	    (mew-pop-message2 pnm "1 message retrieved" left)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message2 pnm (format "%d messages retrieved" rttl) left)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'scan)
+	  (mew-biff-clear)
+	  (cond
+	   ((or (= rttl 0) (null uidl))
+	    (mew-pop-message pnm "No messages scanned"))
+	   ((= rttl 1)
+	    (mew-pop-message pnm "1 message scanned")
+	    (mew-lisp-save file uidl nil 'unlimit)
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-pop-message pnm "%d messages scanned" rttl)
+	    (mew-lisp-save file uidl nil 'unlimit)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'exec)
+	  (when kils
+	    (mew-mark-exec-unlink bnm kils)
+	    (mew-mark-kill-invisible)
+	    (mew-summary-folder-cache-save))
+	  (cond
+	   ((= dttl 0)
+	    (mew-pop-message pnm "No messages deleted"))
+	   ((= dttl 1)
+	    (mew-pop-message pnm "1 message deleted"))
+	   (t
+	    (mew-pop-message pnm "%d messages deleted" dttl))))))
+      ;;
+      (and mew-use-async-write (mew-unix-sync))
+      (mew-net-status-clear (mew-pop-get-status-buf pnm))
+      (mew-info-clean-up pnm)
+      (set-buffer-modified-p nil)
+      (mew-summary-unlock)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (mew-net-disp-info-display disp-info)
+      (unless (eq directive 'biff)
+	(run-hooks 'mew-pop-sentinel-non-biff-hook))
+      (run-hooks 'mew-pop-sentinel-hook)
+      (when (and mew-auto-flush-queue flush)
+	(mew-smtp-flush-queue mew-case)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -934,8 +939,7 @@
 	 (tag (mew-pop-passtag2 case))
 	 passwd)
     (when (get-buffer case:inbox)
-      (save-excursion
-	(set-buffer case:inbox)
+      (with-current-buffer case:inbox
 	(when (and (mew-summary-exclusive-p 'no-msg)
 		   (and (or mew-use-cached-passwd mew-use-master-passwd)
 			(setq passwd (mew-passwd-get-passwd tag))))
@@ -948,8 +952,7 @@
 	 (inbox (mew-proto-inbox-folder nil case))
 	 (case:inbox (mew-case-folder case inbox)))
     (when (get-buffer case:inbox)
-      (save-excursion
-	(set-buffer case:inbox)
+      (with-current-buffer case:inbox
 	(when (mew-summary-exclusive-p)
 	  (mew-pop-retrieve case 'biff case:inbox))))))
 
@@ -957,13 +960,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2007 Mew developing team.
+;; Copyright (C) 1999-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -972,7 +975,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
