@@ -1,11 +1,13 @@
 ;;; mew-scan.el --- Scanning messages for Mew
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Oct  2, 1996
 
 ;;; Code:
 
 (require 'mew)
+(eval-when-compile
+  (require 'mew-env0))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -849,14 +851,15 @@ non-nil, only headers of messages are cached. If executed with
 
 (defun mew-summary-folder-cache-save ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-file folder mew-summary-cache-file)))
+	 (cache (mew-expand-file folder mew-summary-cache-file))
+	 (cbuf (current-buffer)))
     (when (file-writable-p cache)
       (mew-touch-folder folder)
       (save-restriction
 	(widen)
-	(if (mew-decode-syntax-p)
-	    (let ((cbuf (current-buffer))
-		  (min (point-min))
+	(if (and (mew-decode-syntax-p)
+		 (equal (mew-decode-syntax-buffer) cbuf))
+	    (let ((min (point-min))
 		  (max (point-max))
 		  (beg (mew-decode-syntax-begin))
 		  (end (mew-decode-syntax-end)))
@@ -898,7 +901,7 @@ non-nil, only headers of messages are cached. If executed with
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2015 Mew developing team.
+;; Copyright (C) 1996-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
