@@ -1,11 +1,13 @@
 ;;; mew-summary4.el --- Summary mode for Mew
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Oct  2, 1996
 
 ;;; Code:
 
 (require 'mew)
+(eval-when-compile
+  (require 'mew-env0))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -19,21 +21,18 @@
 ;;; Switching to folder
 ;;;
 
-(defun mew-summary-switch-to-folder (case:folder &optional p-folder func)
-  (let ((ofolder (mew-summary-folder-name 'ext))
-	new-folder)
+(defun mew-summary-switch-to-folder (case:folder &optional func)
+  (let ((ofolder (mew-summary-folder-name 'ext)))
     (cond
      ((get-buffer case:folder)
       (switch-to-buffer case:folder)
       (mew-set-buffer-cs mew-cs-m17n)
-      (when (and p-folder (mew-folder-virtualp case:folder))
-	(mew-vinfo-set-parent-folder p-folder))
       (unless (mew-folder-virtualp case:folder) (mew-summary-folder-cache-load))
-      (unless (string= ofolder case:folder) (mew-window-configure 'summary)))
+      (unless (string= ofolder case:folder) (mew-window-configure 'summary))
+      nil) ;; existing folder
      (t
-      (setq new-folder t)
-      (unless func (setq func 'switch-to-buffer))
-      (funcall func (get-buffer-create case:folder))
+      (funcall (or func 'switch-to-buffer) (get-buffer-create case:folder))
+      (kill-all-local-variables)
       (mew-set-buffer-cs mew-cs-m17n)
       (mew-buffers-setup case:folder)
       (let* ((case (mew-case:folder-case case:folder))
@@ -43,14 +42,12 @@
 	(mew-sinfo-set-folder folder)
 	(mew-sinfo-set-inboxp inboxp))
       (if (mew-folder-virtualp case:folder)
-	  (progn
-	    (if p-folder (mew-vinfo-set-parent-folder p-folder))
-	    (mew-virtual-mode))
+	  (mew-virtual-mode)
 	(mew-summary-mode)
 	(if mew-summary-trace-directory (cd (mew-expand-folder case:folder)))
 	(mew-summary-folder-cache-load))
-      (mew-window-configure 'summary)))
-    new-folder))
+      (mew-window-configure 'summary)
+      t)))) ;; new-folder
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -63,12 +60,10 @@
 Summary mode."
   (interactive "P")
   (let* ((proto (mew-proto-to-go (mew-summary-folder-name 'ext)))
-	 (case1 (if (mew-folder-remotep proto)
-		    (mew-sinfo-get-case) ;; a remote folder
-		  mew-case)) ;; a local/virtual folder
+	 (case1 (car (mew-summary-case-proto)))
 	 (inbox (mew-proto-inbox-folder	proto case1))
 	 (case:folder (mew-input-folder case1 inbox))
-	 case folder buf win frame folder-alist sep)
+	 case folder buf win frame folder-alist)
     (if (null case:folder)
 	(message "The folder does not exist")
       (setq case (mew-case:folder-case case:folder))
@@ -76,16 +71,16 @@ Summary mode."
       (cond
        ((mew-folder-imapp folder)
 	(setq folder-alist (mew-imap-folder-alist case))
-	(setq sep (mew-imap-separator case))
+	;;
 	(unless (or (assoc folder folder-alist)
-		    (assoc (concat folder sep) folder-alist))
+		    (assoc (concat folder (mew-imap-separator case)) folder-alist))
 	  (message "\"%s\" does not exist. %s" case:folder
-		   (mew-substitute-for-summary "Type '3\\[mew-status-update]' to collect IMAP folders"))
+		   (mew-substitute-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect IMAP folders"))
 	  (setq case:folder nil)))
        ((mew-folder-nntpp folder)
 	(unless (assoc folder (mew-nntp-folder-alist case))
 	  (message "\"%s\" does not exist. %s" case:folder
-		   (mew-substitute-for-summary "Type '2\\[mew-status-update]' to collect newsgroups"))
+		   (mew-substitute-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect newsgroups"))
 	  (setq case:folder nil))))
       (when case:folder
 	(when mew-use-other-frame-for-summary
@@ -96,11 +91,7 @@ Summary mode."
 		    (raise-frame frame)
 		    (select-frame frame)
 		    ;; Ensure, if possible, that frame gets input focus.
-		    (if (eq window-system 'w32)
-			(w32-focus-frame frame)
-		      (when focus-follows-mouse
-			(set-mouse-position
-			 (selected-frame) (1- (frame-width)) 0))))
+		    (mew-focus-frame frame))
 		(select-frame (make-frame)))
 	    (select-frame (make-frame))))
 	(mew-summary-visit-folder case:folder goend)))))
@@ -160,10 +151,25 @@ the message is then displayed."
 	      (mew-summary-display))))
     (let ((line (string-to-number (read-string "Line No.: " ""))))
       (when (/= line 0)
-	(goto-line line)
+	(goto-char (point-min)) (forward-line (1- line))
 	(if mew-summary-goto-line-then-display
 	    (mew-summary-display))))))
 
+(defun mew-summary-goto-original-message ()
+  "Go to the original message in the original Summary mode"
+  (interactive)
+  (cond
+   ((mew-summary-p)
+    (message "This is the original"))
+   ((mew-virtual-p)
+    (let ((folder (mew-folder-path-to-folder (mew-summary-folder-name) 'has-proto))
+	  (msg (mew-summary-message-number)))
+      (when folder
+	(mew-summary-visit-folder folder nil 'no-ls)
+	(if (mew-summary-search-msg msg)
+	    (mew-summary-display)
+	  (message "No original message found")))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Moving to Message mode
@@ -173,8 +179,7 @@ the message is then displayed."
   "Go to Header mode."
   (interactive)
   (let (msgp)
-    (save-excursion
-      (set-buffer (window-buffer (next-window)))
+    (with-current-buffer (window-buffer (next-window))
       (if (mew-message-p) (setq msgp t)))
     (if msgp
 	(other-window 1)
@@ -190,11 +195,9 @@ the message is then displayed."
 	 (regex (mew-folder-regex draft-dir))
          (bufs (mew-buffer-list regex))
 	 (msgregex (format "\\([0-9]+\\)\\(%s\\)?$" mew-suffix))
-         nums num n)
+         nums n)
     (setq nums (mapcar (lambda (x) (string-match msgregex x) (string-to-number (mew-match-string 1 x))) bufs))
-    (while nums
-      (setq num (car nums))
-      (setq nums (cdr nums))
+    (dolist (num nums)
       (if (or (null n) (< n num))
 	  (setq n num)))
     (and n (format "%s%d%s" draft-dir n (if mew-use-suffix mew-suffix "")))))
@@ -217,7 +220,7 @@ the message is then displayed."
 
 (defun mew-summary-jump-top ()
   "Go to the beginning of this Summary mode.
-If 'mew-summary-jump-top-then-display' is non-nil, 
+If 'mew-summary-jump-top-then-display' is non-nil,
 the top message is then displayed."
   (interactive)
   (if mew-use-push-mark (mew-push-mark))
@@ -227,12 +230,15 @@ the top message is then displayed."
 
 (defun mew-summary-jump-bottom ()
   "Go to the end of this Summary mode.
-If 'mew-summary-jump-bottom-then-display' is non-nil, 
+If 'mew-summary-jump-bottom-then-display' is non-nil,
 the top message is then displayed."
   (interactive)
   (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-max))
-  (unless (bobp) (forward-line -1))
+  (if (and (mew-summary-exclusive-p 'no-msg)
+	   (not (bobp))
+	   (mew-sinfo-get-disp-msg))
+      (forward-line -1))
   (if mew-summary-jump-bottom-then-display
       (mew-summary-display)))
 
@@ -293,13 +299,12 @@ the top message is then displayed."
 If called with '\\[universal-argument]', the body of the message
 \(excluding its header) is sent."
   (interactive
-   (list current-prefix-arg 
+   (list current-prefix-arg
 	 (read-string "Shell command on message: " mew-last-shell-command)))
   (mew-summary-display 'redisplay)
   (when (or (not mew-ask-pipe)
             (y-or-n-p "Send this message to pipe? "))
-    (save-excursion
-      (set-buffer (mew-buffer-message))
+    (with-current-buffer (mew-buffer-message)
       (save-restriction
 	(widen)
 	(if (string= command "") (setq command mew-last-shell-command))
@@ -323,8 +328,7 @@ If called with '\\[universal-argument]', you can specify a printer name."
 			  (read-string "Printer name: ")
 			printer-name))
 	have-header str)
-    (save-excursion
-      (set-buffer (mew-buffer-message))
+    (with-current-buffer (mew-buffer-message)
       (save-restriction
 	(widen)
 	(setq have-header (mew-msghdr-p))
@@ -336,6 +340,32 @@ If called with '\\[universal-argument]', you can specify a printer name."
 	    (mew-header-delete-other-lines mew-field-for-printing))
 	(funcall mew-print-function)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Saving a message to a file with header decoded
+;;;
+
+(defun mew-summary-store (&optional askcs)
+  "Saving a buffer of Message mode to a file.
+If executed with '\\[universal-argument]', coding-system is asked."
+  (interactive "P")
+  (mew-summary-display 'redisplay)
+  (let* ((file (mew-summary-input-file-name))
+	 (doit (if (file-exists-p file)
+		   (if (y-or-n-p (format "File exists. Override it? "))
+		       t
+		     nil)
+		 t)))
+    (if (not doit)
+	(message "Not saved")
+      (let ((writecs (if askcs
+			 (read-coding-system "Coding-system: ")
+		       (default-value 'buffer-file-coding-system))))
+	(with-current-buffer (mew-buffer-message)
+	  (mew-frwlet mew-cs-dummy writecs
+	    ;; do not specify 'no-msg
+	    (write-region (point-min) (point-max) file)))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; X-Face:
@@ -346,16 +376,14 @@ If called with '\\[universal-argument]', you can specify a printer name."
   (interactive)
   (mew-summary-msg
    (let ((file (mew-make-temp-name)) xface)
-     (save-excursion
-       (set-buffer (mew-buffer-message))
+     (with-current-buffer (mew-buffer-message)
        (setq xface (mew-header-get-value mew-x-face:)))
      (when xface
        (with-temp-buffer
 	 (insert xface)
 	 (mew-x-face-compface-to-xbm)
-	 (mew-frwlet
-	  mew-cs-dummy mew-cs-text-for-write
-	  (write-region (point-min) (point-max) file nil 'no-msg)))
+	 (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	   (write-region (point-min) (point-max) file nil 'no-msg)))
        (mew-mime-image-ext file)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -372,7 +400,7 @@ argument."
 	  (msg (mew-summary-message-number))
 	  (file (mew-expand-msg fld msg))
 	  (cmd-args (mew-input-command))
-	  (cmd (car cmd-args)) 
+	  (cmd (car cmd-args))
 	  (args (nconc (cdr cmd-args) (list file))))
      (message "Executing %s for %s..." cmd msg)
      (apply 'call-process cmd nil nil nil args)
@@ -384,7 +412,7 @@ marked with '*' as arguments."
   (interactive)
   (mew-summary-multi-msgs
    (let* ((cmd-args (mew-input-command))
-	  (cmd (car cmd-args)) 
+	  (cmd (car cmd-args))
 	  (args (nconc (cdr cmd-args) FILES)))
      (message "Executing %s ..." cmd)
      (apply 'call-process cmd nil nil nil args)
@@ -403,196 +431,10 @@ marked with '*' as arguments."
 	(message "No draft buffer exists!")
       (setq buf (mew-input-draft-buffer draft))
       (if (get-buffer buf)
-	  (save-excursion
-	    (set-buffer buf)
+	  (with-current-buffer buf
 	    (mew-draft-cite arg))
 	(message "No such draft buffer!")))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Executing external commands
-;;;
-
-(defun mew-summary-ask-ct (ct fname)
-  (let (pt fl)
-    (cond
-     (fname
-      (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
-		   (mew-content-type (mew-sinfo-get-case))))
-      (setq pt "Type for %s (%s): ")
-      (setq fl fname))
-     (t
-      (setq pt "Type %s(%s): ")
-      (setq fl "")))
-    (mew-input-type pt fl ct mew-mime-content-type-list)))
-
-(defun mew-summary-execute-external (&optional ask-command)
-  "Execute an external command according to Content-Type:.
-If this command is executed on the entire message, the first part
-is chosen as a target. 
-
-If Content-Type of the target part is Application/Octet-Stream,
-Content-Type is automatically asked. And if it has both 
-internal and external visualization mechanisms, you are asked
-which you want to use.
-
-If executed with '\\[universal-argument]', you can specify a
-command to be executed.
-
-See 'mew-mime-content-type' to know how actions can be defined."
-  (interactive "P")
-  (mew-summary-msg-or-part
-   (let* ((fld (mew-summary-folder-name))
-	  (msg (mew-summary-message-number2))
-	  (nums (mew-syntax-nums))
-	  (cache (mew-cache-hit fld msg 'must-hit))
-	  (syntax (mew-cache-decode-syntax cache))
-	  (stx (mew-syntax-get-entry syntax nums))
-	  (ctl (mew-syntax-get-ct stx))
-	  (ct (mew-syntax-get-value ctl 'cap))
-	  (win (selected-window))
-	  begin end params cdpl fname program options async was-apo erase-p
-	  pro-opt ent1 ent2)
-     (when (and (string= ct mew-ct-msg)
-		(or (not ask-command)
-		    (not (y-or-n-p "Save the entire message (y) or the first part (n)? "))))
-       (setq stx (mew-syntax-get-part stx))
-       (if (mew-syntax-multipart-p stx)
-	   (setq stx (mew-syntax-get-part stx)))
-       (setq ctl (mew-syntax-get-ct stx))
-       (setq ct (mew-syntax-get-value ctl 'cap)))
-     (setq begin (mew-syntax-get-begin stx))
-     (setq end (mew-syntax-get-end stx))
-     (setq params (mew-syntax-get-params ctl))
-     (setq cdpl (mew-syntax-get-cdp stx))
-     (setq fname (mew-syntax-get-filename cdpl ctl))
-     (when (or (string= ct mew-ct-apo)
-	       (eq (mew-ctdb-prog (mew-ctdb-by-ct ct))
-		   'mew-mime-application/octet-stream))
-       (setq ct (mew-summary-ask-ct ct fname))
-       (setq was-apo t))
-     (if (not ask-command)
-	 (setq program (mew-ctdb-prog (mew-ctdb-by-ct ct)))
-       (setq pro-opt (mew-input-command mew-default-external-program))
-       (setq program (car pro-opt))
-       (setq options (cdr pro-opt)))
-     ;; prog
-     ;; func => called for both
-     ;; (prog ...)
-     ;; (nil prog) == prog
-     ;; (nil (prog...)) == (prog...)
-     ;; (nil func) != func
-     ;; (func func) => need to select
-     ;; (func prog) => need to select
-     ;; (func (prog ...)) => need to select
-     (cond
-      ((stringp program)
-       (setq async t))
-      ((symbolp program)
-       (setq erase-p t))
-      ((listp program)
-       (setq ent1 (nth 0 program))
-       (setq ent2 (nth 1 program))
-       (cond
-	((stringp ent1)
-	 (mew-set '(program options async) program))
-	((null ent1)
-	 (cond
-	  ((stringp ent2)
-	   (setq program ent2)
-	   (setq async t))
-	  ((listp ent2)
-	   (mew-set '(program options async) ent2))
-	  (t ;; symbol
-	   (setq program ent2))))
-	(t ;; symbol
-	 (if (and was-apo
-		  (y-or-n-p "Internal (y) or External (n)? "))
-	     (progn
-	       (setq erase-p t)
-	       (setq program ent1))
-	   (setq program ent2))
-	 (cond
-	  ((stringp program)
-	   (setq async t))
-	  ((listp program)
-	   (mew-set '(program options async) program)))))))
-     (mew-summary-toggle-disp-msg 'on)
-     (mew-window-configure 'message)
-     (if erase-p (mew-erase-buffer))
-     ;; message buffer
-     (unwind-protect
-	 (mew-elet
-	  (cond
-	   ((stringp program)
-	    (mew-summary-execute-program
-	     program ct ctl cache begin end params fname options async))
-	   ((symbolp program)
-	    (mew-summary-execute-symbol
-	     program ct ctl cache begin end params fname was-apo)))
-	  (mew-summary-display-postscript 'no-hook))
-       (select-window win)))))
-
-(defun mew-summary-execute-symbol (program ct ctl cache begin end params fname was-apo)
-  (cond
-   ((not (fboundp program))
-    (message "%s is not implemented" (symbol-name program)))
-   ((eq program mew-prog-rfc822)
-    (message "%s cannot be executed" mew-prog-rfc822))
-   ((or (mew-ct-imagep ct) (mew-ct-modelp ct))
-    (funcall program cache begin end params fname ct))
-   (t
-    (let ((mew-use-text/html t)
-	  (mbuf (current-buffer))
-	  tbuf)
-      (if (and was-apo (mew-ct-textp ct)) ;; decode-broken?
-	  (with-temp-buffer
-	    (insert-buffer-substring cache begin end)
-	    (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv)
-	    (setq tbuf (current-buffer))
-	    (setq begin (point-min))
-	    (setq end (point-max))
-	    (set-buffer mbuf)
-	    (funcall program tbuf begin end params))
-	(funcall program cache begin end params))))))
-
-;;; external
-(defun mew-summary-execute-program (program ct ctl cache begin end params fname options async)
-  (if (not (mew-which-exec program))
-      (message "%s does not exist" program)
-    (let ((file (mew-make-temp-name fname))
-	  wcs)
-      (save-excursion
-	(set-buffer cache)
-	;; NEVER use call-process-region for privacy reasons
-	(cond
-	 ((not (mew-ct-linebasep ct))
-	  (setq wcs mew-cs-binary))
-	 ((not (mew-ct-textp ct))
-	  (setq wcs mew-cs-text-for-write))
-	 (t
-	  (cond
-	   ((or (string= mew-ct-htm ct) (string= mew-ct-xml ct))
-	    (setq wcs (mew-text/html-detect-cs begin end))
-	    (unless (mew-coding-system-p wcs)
-	      (setq wcs (mew-charset-to-cs
-			 (mew-syntax-get-param ctl "charset")))))
-	   (t
-	    (setq wcs (mew-charset-to-cs
-		       (mew-syntax-get-param ctl "charset")))))
-	  (unless (mew-coding-system-p wcs)
-	    (setq wcs (if mew-decode-broken
-			  (mew-charset-to-cs
-			   (mew-charset-guess-region
-			    begin end))
-			mew-cs-text-for-write)))))
-	(mew-frwlet
-	 mew-cs-dummy wcs
-	 (write-region begin end file nil 'no-msg))
-	(if async
-	    (mew-mime-start-process program options file)
-	  (mew-mime-call-process program options file))))))
-  
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Flushing the queue
@@ -634,10 +476,20 @@ If executed with '\\[universal-argument]', you can set the sending case."
 ;;; Unlocking
 ;;;
 
+(defun mew-subprocess-init ()
+  (add-hook 'kill-emacs-hook 'mew-subprocess-kill))
+
+(defun mew-subprocess-clean-up ()
+  (mew-summary-kill-subprocess t)
+  (remove-hook 'kill-emacs-hook 'mew-subprocess-kil))
+
+(defun mew-subprocess-kill ()
+  (mew-summary-kill-subprocess t))
+
 (defun mew-summary-kill-subprocess (&optional kill-ssx)
   "\\<mew-summary-mode-map>
 Kill a process in Summary mode.
-Sometime a process accidentally remains in Summary mode. 
+Sometime a process accidentally remains in Summary mode.
 In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
 Use this command to solve this problem.
 
@@ -645,15 +497,14 @@ If called with '\\[universal-argument]', all SSH processes, if
 any, are killed."
   (interactive "P")
   (let ((process mew-summary-buffer-process)
-	(disp-msg (interactive-p))
+	(disp-msg (mew-called-interactively-p))
 	pros sentinel)
     (if kill-ssx
 	(progn
 	  (setq pros (process-list))
-	  (while pros
-	    (if (string-match "mew" (process-name (car pros)))
-		(delete-process (car pros)))
-	    (setq pros (cdr pros)))
+	  (dolist (pro pros)
+	    (if (string-match "mew" (process-name pro))
+		(delete-process pro)))
 	  (if disp-msg (message "All SSH/SSL/TLS processes were killed")))
       (cond
        ((not (processp process))
@@ -709,19 +560,17 @@ any, are killed."
 	   (case-fold-search nil)
 	   (files FILES) start)
        (with-temp-buffer
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  (while files
-	    (mew-erase-buffer)
-	    (mew-insert-file-contents (car files))
-	    (goto-char (point-min))
-	    (if (re-search-forward mew-eoh nil t)
-		(forward-line))
-	    (setq start (point))
-	    (goto-char (point-max))
-	    (unless (bolp) (insert "\n"))
-	    (write-region start (point-max) tmp 'append 'no-msg)
-	    (setq files (cdr files))))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   (dolist (file files)
+	     (mew-erase-buffer)
+	     (mew-insert-file-contents file)
+	     (goto-char (point-min))
+	     (if (re-search-forward mew-eoh nil t)
+		 (forward-line))
+	     (setq start (point))
+	     (goto-char (point-max))
+	     (unless (bolp) (insert "\n"))
+	     (write-region start (point-max) tmp 'append 'no-msg)))
 	 (mew-erase-buffer)
 	 (cd dir)
 	 ;; mew-folder-local is dummy to let mewencode use
@@ -768,19 +617,17 @@ message."
 	   (folder (mew-input-burst-folder))
 	   (tfile (mew-make-temp-name))
 	   (targets FLD-MSG-LIST)
-	   fld-msg num ct med dbl vec len idx ttl TTL fid FID i ent
+	   num ct med dbl vec len idx ttl TTL fid FID i
 	   beg regex)
        (message "Joining...")
        (with-temp-buffer
 	 (mew-set-buffer-multibyte t)
 	 (insert "CD: " (mew-expand-folder2 cfld) "\n")
-	 (while targets
-	   (setq fld-msg (car targets))
-	   (setq targets (cdr targets))
-	   (insert (cdr fld-msg) "\n"))
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  (write-region (point-min) (point-max) tfile nil 'no-msg))
+	 (let ((default-directory (mew-expand-folder cfld)))
+	   (dolist (fld-msg targets)
+	     (insert (mew-msg-get-filename (cdr fld-msg)) "\n")))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   (write-region (point-min) (point-max) tfile nil 'no-msg))
 	 (mew-erase-buffer)
 	 (call-process mew-prog-mewl nil t nil
 		       "-b" mew-mail-path "-l" "0"
@@ -800,9 +647,7 @@ message."
 	     (setq dbl (cons (cons num ct) dbl))))
 	 (setq len (length dbl))
 	 (setq vec (make-vector len nil))
-	 (while dbl
-	   (setq ent (car dbl))
-	   (setq dbl (cdr dbl))
+	 (dolist (ent dbl)
 	   (setq num (car ent))
 	   (setq ttl (mew-syntax-get-param ent "total"))
 	   (setq fid (mew-syntax-get-param ent "id"))
@@ -821,61 +666,61 @@ message."
 	       (aset vec idx num)
 	     (error "invalid number")))
 	 (unless TTL (error "no total"))
-	 (setq i 0)
-	 (while (< i len)
-	   (unless (stringp (aref vec i)) (error "Not enough fragments"))
-	   (setq i (1+ i)))
+	 (dotimes (i len)
+	   (unless (stringp (aref vec i)) (error "Not enough fragments")))
 	 ;; now reassemble
 	 (mew-erase-buffer)
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  ;; the first fragment
-	  (goto-char (point-max))
-	  (save-restriction
-	    (narrow-to-region (point) (point))
-	    (mew-insert-file-contents (mew-expand-msg cfld (aref vec 0)))
-	    ;; Removing unnecessary fields from the encapsulating
-	    ;; (outer) header.
-	    (goto-char (point-min))
-	    (mew-header-delete-lines mew-field-delete-for-joining)
-	    ;; Concatenating the two headers.
-	    (goto-char (point-min))
-	    (re-search-forward mew-eoh nil t)
-	    (setq beg (point))
-	    (while (looking-at mew-eoh) (forward-line))
-	    (delete-region beg (point))
-	    ;; Removing unnecessary fields from the encapsulated
-	    ;; (inner) header.
-	    (setq beg (point))
-	    (when (re-search-forward mew-eoh nil t)
-	      (setq regex (mew-make-field-regex mew-field-delete-for-joining))
-	      (save-restriction
-		(narrow-to-region beg (point))
-		(goto-char (point-min))
-		(let ((case-fold-search t))
-		  (while (not (eobp))
-		    (if (looking-at regex)
-			(setq beg nil) ;; logic is reversed
-		      (setq beg (point)))
-		    (forward-line)
-		    (mew-header-goto-next)
-		    (if beg (delete-region beg (point))))))))
-	  ;; the second and subsequent fragments.
-	  (setq i 1)
-	  (while (< i len)
-	    (goto-char (point-max))
-	    (save-restriction
-	      (narrow-to-region (point) (point))
-	      (mew-insert-file-contents (mew-expand-msg cfld (aref vec i)))
-	      (goto-char (point-min))
-	      (re-search-forward mew-eoh nil t)
-	      (forward-line)
-	      (delete-region (point-min) (point)))
-	    (setq i (1+ i)))
-	  (mew-header-delete-lines `(,mew-x-mew-uidl:))
-	  (write-region (point-min) (point-max)
-			(mew-folder-new-message folder)
-			nil 'no-msg)))
+	 (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	   ;; the first fragment
+	   (goto-char (point-max))
+	   (save-restriction
+	     (narrow-to-region (point) (point))
+	     (mew-insert-file-contents (mew-expand-msg cfld (aref vec 0)))
+	     ;; Removing unnecessary fields from the encapsulating
+	     ;; (outer) header.
+	     (goto-char (point-min))
+	     (mew-header-delete-lines mew-field-delete-for-joining)
+	     ;; Concatenating the two headers.
+	     (goto-char (point-min))
+	     (re-search-forward mew-eoh nil t)
+	     (setq beg (point))
+	     (while (looking-at mew-eoh)
+	       (forward-line)
+	       (if (eobp) (error "invalid message (number 1)")))
+	     (delete-region beg (point))
+	     ;; Removing unnecessary fields from the encapsulated
+	     ;; (inner) header.
+	     (setq beg (point))
+	     (when (re-search-forward mew-eoh nil t)
+	       (setq regex (mew-make-field-regex mew-field-delete-for-joining))
+	       (save-restriction
+		 (narrow-to-region beg (point))
+		 (goto-char (point-min))
+		 (let ((case-fold-search t))
+		   (while (not (eobp))
+		     (if (looking-at regex)
+			 (setq beg nil) ;; logic is reversed
+		       (setq beg (point)))
+		     (forward-line)
+		     (mew-header-goto-next)
+		     (if beg (delete-region beg (point))))))))
+	   ;; the second and subsequent fragments.
+	   (setq i 1)
+	   (while (< i len)
+	     (goto-char (point-max))
+	     (save-restriction
+	       (narrow-to-region (point) (point))
+	       (mew-insert-file-contents (mew-expand-msg cfld (aref vec i)))
+	       (goto-char (point-min))
+	       (re-search-forward mew-eoh nil t)
+	       (forward-line)
+	       (if (eobp) (error "invalid message (number %d)" (1+ i)))
+	       (delete-region (point-min) (point)))
+	     (setq i (1+ i)))
+	   (mew-header-delete-lines `(,mew-x-mew-uidl:))
+	   (write-region (point-min) (point-max)
+			 (mew-folder-new-message folder)
+			 nil 'no-msg)))
        (mew-touch-folder folder)
        (message "Joining...done")
        (when mew-ask-folder-after-join
@@ -914,8 +759,7 @@ message."
 	  (if (string= folder trash) (mew-summary-reset))
 	  (message "Removing all messages in %s..." case:trash)
 	  (when (get-buffer case:trash)
-	    (save-excursion
-	      (set-buffer case:trash)
+	    (with-current-buffer case:trash
 	      (mew-summary-kill-subprocess)))
 	  (mew-summary-unlink-msgs case:trash msgs)
 	  (mew-summary-folder-cache-clean case:trash)
@@ -923,8 +767,11 @@ message."
 	  (when (mew-folder-imapp trash)
 	    (if (get-buffer case:trash)
 		(set-buffer case:trash)
-	      (mew-summary-switch-to-folder case:trash nil 'set-buffer))
-	    (mew-imap-retrieve case 'exec case:trash '(("1:*") nil nil nil 0 -1))))))))
+	      (mew-summary-switch-to-folder case:trash 'set-buffer))
+	    (mew-imap-retrieve case 'exec case:trash
+			       (mew-make-execinfo :remove '("1:*")
+						  :rttl   0
+						  :dttl   -1))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -985,8 +832,7 @@ the region are handled."
 	 (setq dstmsg (mew-folder-new-message dstfld 'numonly)))
        (setq msgs (mew-summary-mark-collect mew-mark-review beg end))
        (when (get-buffer dstfld)
-	 (save-excursion
-	   (set-buffer dstfld)
+	 (with-current-buffer dstfld
 	   (setq exclusivep (mew-summary-exclusive-p))))
        (cond
 	((null msgs)
@@ -997,26 +843,24 @@ the region are handled."
 	 (message "Copying...")
 	 (setq total (length msgs))
 	 (setq i 1)
-	 (while msgs
-	   (setq file (mew-expand-msg fld (car msgs)))
+	 (dolist (msg msgs)
+	   (setq file (mew-expand-msg fld msg))
 	   (setq dstfile (mew-expand-new-msg dstfld dstmsg))
 	   (message "Copying (%d/%d)..." i total)
 	   (mew-summary-local-copy-one file dstfile)
 	   (setq dstmsg (number-to-string (1+ (string-to-number dstmsg))))
-	   (setq msgs (cdr msgs))
 	   (setq i (1+ i)))
 	 (mew-touch-folder dstfld)
 	 (message "Copying...done")))))))
 
 (defun mew-summary-local-copy-one (srcfile dstfile)
   (with-temp-buffer
-    (mew-frwlet
-     mew-cs-text-for-read mew-cs-text-for-write
-     (mew-insert-file-contents srcfile)
-     (goto-char (point-min))
-     (mew-header-delete-lines (list mew-x-mew-uidl:))
-     (write-region (point-min) (point-max) dstfile nil 'no-msg)
-     (mew-set-file-modes dstfile))))
+    (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+      (mew-insert-file-contents srcfile)
+      (goto-char (point-min))
+      (mew-header-delete-lines (list mew-x-mew-uidl:))
+      (write-region (point-min) (point-max) dstfile nil 'no-msg)
+      (mew-set-file-modes dstfile))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1040,7 +884,7 @@ the region are handled."
 	 (when (y-or-n-p "Delete this folder? ")
 	   (mew-delete-directory-recursively dir)
 	   (mew-local-folder-delete folder)
-	   (mew-nmz-folder-index-delete folder)
+	   (run-hook-with-args 'mew-summary-delete-folder-hook folder)
 	   (mew-summary-visit-folder mew-inbox-folder)
 	   (mew-kill-buffer buf))))
       ((mew-folder-imapp folder)
@@ -1057,7 +901,7 @@ the region are handled."
 		 (setq mew-summary-buffer-process-error nil))
 	     (mew-imap-folder-delete case folder)
 	     (mew-delete-directory-recursively dir) ;; cache
-	     (mew-nmz-folder-index-delete case:folder)
+	     (run-hook-with-args 'mew-summary-delete-folder-hook case:folder)
 	     (mew-summary-visit-folder mew-imap-inbox-folder)
 	     (mew-kill-buffer buf)))))))))
 
@@ -1080,9 +924,9 @@ the region are handled."
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
 	((file-exists-p new-dir)
-	 (message "%s was not moved so %s is existed" folder new-folder))
+	 (message "%s was not moved since %s exists" folder new-folder))
 	(t
-	 (rename-file dir new-dir)
+	 (mew-rename-directory dir new-dir)
 	 (mew-summary-switch-to-folder new-folder)
 	 (mew-kill-buffer buf)
 	 (if (mew-folder-node-p folder)
@@ -1090,7 +934,7 @@ the region are handled."
 	     (mew-local-update 'force)
 	   (mew-local-folder-delete folder)
 	   (mew-local-folder-insert new-folder))
-	 (mew-nmz-folder-index-rename folder new-folder))))
+	 (run-hook-with-args 'mew-summary-rename-folder-hook folder new-folder))))
       ((mew-folder-imapp folder)
        (setq new-folder (car (mew-input-refile-folders (list folder) t case mew-folder-imap)))
        (setq case:new-folder (mew-case-folder case new-folder))
@@ -1099,7 +943,7 @@ the region are handled."
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
 	((file-exists-p new-dir)
-	 (message "%s was not moved so %s is existed" folder new-folder))
+	 (message "%s was not moved since %s exists" folder new-folder))
 	(t
 	 (mew-imap-retrieve case 'rename case:folder case:new-folder)
 	 (mew-timing)
@@ -1112,12 +956,13 @@ the region are handled."
 	   (mew-summary-switch-to-folder case:new-folder)
 	   (mew-kill-buffer buf)
 	   (if (mew-folder-node-p folder case)
-	       (if (y-or-n-p "Collect a folder list now? ")
+	       (if (y-or-n-p "Collect folders now? ")
 		   (mew-imap-update case)
-		 (mew-message-for-summary "Type '3\\[mew-status-update]' to collect a folder list"))
+		 (mew-message-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect folders"))
 	     (mew-imap-folder-delete case folder)
 	     (mew-imap-folder-insert case new-folder))
-	   (mew-nmz-folder-index-rename case:folder case:new-folder)))))))))
+	   (run-hook-with-args 'mew-summary-rename-folder-hook
+			       case:folder case:new-folder)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1130,11 +975,11 @@ the region are handled."
 ;;;
 
 (defun mew-summary-toggle-disp-msg (&optional arg)
-  "Toggle 'Summary mode only' and 'Summary & Message mode'. If 
+  "Toggle 'Summary mode only' and 'Summary & Message mode'. If
 you choose 'Summary mode only', you can quickly put the delete
 	marks since the next message is not displayed."
   (interactive)
-  (cond 
+  (cond
    ((eq arg 'on)
     (mew-sinfo-set-disp-msg t))
    ((eq arg 'off)
@@ -1260,7 +1105,7 @@ See also 'mew-spam-prog' and 'mew-spam-prog-args."
 	 (if (= (point) beg)
 	     (message "Learning as spam...done")
 	   (setq end (1- (point)))
-	   (message (concat "Learned as spam: " (mew-buffer-substring beg end)))))))))
+	   (message "%s" (concat "Learned as spam: " (mew-buffer-substring beg end)))))))))
 
 (defun mew-summary-learn-ham ()
   "Learn that this message is a ham.
@@ -1284,7 +1129,7 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 	 (if (= (point) beg)
 	     (message "Learning as ham...done")
 	   (setq end (1- (point)))
-	   (message (concat "Learned as ham: " (mew-buffer-substring beg end)))))))))
+	   (message "%s" (concat "Learned as ham: " (mew-buffer-substring beg end)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1292,23 +1137,76 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 ;;;
 
 (defun mew-summary-toggle-invisible ()
+  "Toggle the value of search-invisible.
+Invisible information in Summary mode cannot be searched by default.
+Executing this command enables searching such information."
   (interactive)
   (if (eq search-invisible t)
       (setq search-invisible 'open)
     (setq search-invisible t))
   (message "'search-invisible' is set to '%s'" search-invisible))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Lines
+;;;
+
+(defun mew-summary-line ()
+  "Toggle normal lines, wrapped lines and long lines."
+  (interactive)
+  (let ((fld (mew-summary-folder-name))
+	(msg (mew-summary-message-number))
+	(cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (progn
+	    (mew-message-line)
+	    (mew-message-mode-line fld msg))
+	(select-window cwin)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Horizontal Scrolling
+;;;
+
+(defun mew-summary-scroll-left ()
+  "Scroll the Message window to the right."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-left)
+	(select-window cwin)))))
+
+(defun mew-summary-scroll-right ()
+  "Scroll the Message window to the left."
+  (interactive)
+  (let ((cwin (get-buffer-window (current-buffer)))
+	(mwin (get-buffer-window (mew-buffer-message))))
+    (if (not mwin)
+	(message "No message is displayed")
+      (select-window mwin)
+      (unwind-protect
+	  (mew-message-scroll-right)
+	(select-window cwin)))))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1317,7 +1215,7 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
