@@ -1,6 +1,6 @@
 ;;; mew-vars2.el
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Feb  1, 1999
 
 ;;; Code:
@@ -549,6 +549,7 @@ the next version of Mew.")
     ("^DKIM-Signature:$" nil)
     ("^ARC-" nil)
     ("^Received-SPF:$" nil)
+    ("^Autocrypt:$" nil)
     ("^Authentication-Results:$"
      mew-authentication-results
      mew-face-header-important
@@ -939,7 +940,7 @@ imap-proxy-server, imap-proxy-port,
 nntp-server, nntp-port, nntp-ssh-server, nntp-ssl, nntp-ssl-port,
 nntp-user, nntp-size, nntp-header-only,
 nntp-msgid-user, nntp-msgid-domain,
-ssl-cert-directory, ssl-verify-level,
+prog-ssl-arg, ssl-cert-directory, ssl-verify-level,
 inbox-folder, queue-folder, postq-folder,
 mailbox-type, mbox-command, mbox-command-arg,
 signature-file, content-type, refile-guess-alist,
@@ -983,7 +984,7 @@ An example is as follows:
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2015 Mew developing team.
+;; Copyright (C) 2000-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
