@@ -1,8 +1,8 @@
 ;;; mew.el --- Messaging in the Emacs World
 
-;; Author:  Kazu Yamamoto <Kazu@Mew.org>
+;; Author:  Mew developing team
 ;; Created: Mar 23, 1994
-;; Revised: Jan 10, 2007
+;; Revised: Feb 10, 2023
 
 ;;; Commentary:
 
@@ -18,7 +18,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "5.2"
+(defconst mew-version-number "6.9"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -26,16 +26,11 @@
 (require 'mew-const)
 (require 'mew-blvs)
 (require 'mew-func)
-(require 'mew-vars)
+(require 'mew-vars) ;; mew-env && mew-key
 (require 'mew-vars2)
 (require 'mew-vars3)
-(cond
- ((memq system-type '(windows-nt cygwin))
-  (require 'mew-win32))
- ((eq system-type 'darwin)
-  (require 'mew-darwin))
- (t
-  (require 'mew-unix)))
+(eval-when-compile
+  (require 'mew-env0))
 
 (defun mew-version-show ()
   "Show mew-version in minibuffer."
@@ -49,12 +44,9 @@
 
 (defvar mew-debug nil
   "'decode, 'encode, 'net, 'pgp, 'thread, 'sort, t for all.")
-(defsubst mew-debug (category)
+(defun mew-debug (category)
   (or (eq mew-debug t) (eq mew-debug category)))
 
-(defvar mew-profile nil)
-(defvar mew-profile-functions-list nil)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Bootstrap
@@ -67,7 +59,7 @@
   "Execute Mew first unless Mew is running. And retrieve arrived
 messages or just visit to the default folder.
 
-'proto' is determined by 'mew-proto' and 'mew-case'.  
+'proto' is determined by 'mew-proto' and 'mew-case'.
 
 If 'proto' is '+' (ie a local folder), a mailbox is determined
 according to 'mew-mailbox-type'. Otherwise (ie a remote folder), an
@@ -107,9 +99,10 @@ function."
   (mew-summary-send to cc subject))
 
 ;;;###autoload
-(defun mew-user-agent-compose (&optional to subject other-headers continue
-                                             switch-function yank-action
-                                             send-actions)
+(defun mew-user-agent-compose (&optional to subject other-headers _continue
+					 switch-function _yank-action
+					 _send-actions
+					 &rest _dummy)
   "Set up message composition draft with Mew.
 This is 'mail-user-agent' entry point to Mew.
 
@@ -125,13 +118,20 @@ A Draft buffer is prepared according to SWITCH-FUNCTION.
 CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
   (unless mew-init-p (mew-init))
   (let* ((draft (mew-folder-new-message mew-draft-folder))
-	 (attachdir (mew-attachdir draft)))
+	 (attachdir (mew-attachdir draft))
+	 cc asked)
+    (when (and mew-ask-to (null to))
+      (setq to (mew-input-address (concat mew-to: " ")))
+      (setq asked t))
+    (when mew-ask-cc
+      (setq cc (mew-input-address (concat mew-cc: " ")))
+      (setq asked t))
     (mew-current-set-window-config)
     (mew-window-configure 'draft)
     (mew-summary-prepare-draft
      (mew-draft-find-and-switch draft switch-function)
      (mew-delete-directory-recursively attachdir)
-     (mew-draft-header subject nil to nil nil nil nil other-headers)
+     (mew-draft-header subject nil to cc nil nil nil other-headers asked)
      (mew-draft-mode)
      (run-hooks 'mew-draft-mode-newdraft-hook))))
 
@@ -144,6 +144,13 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
   (let ((buf (get-buffer-create "*Mew hello*")))
     (run-hooks 'mew-env-hook)
     (load mew-rc-file 'no-err 'no-msg)
+    (cond
+     ((memq system-type '(windows-nt cygwin))
+      (require 'mew-win32))
+     ((eq system-type 'darwin)
+      (require 'mew-darwin))
+     (t
+      (require 'mew-unix)))
     (require 'mew-varsx)
     (if mew-theme-file (load mew-theme-file 'no-err 'no-msg))
     (switch-to-buffer buf)
@@ -166,14 +173,18 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 	(progn
 	  ;; sanity check
 	  (cond
-	   ((string-match "^\\(18\\|19\\)" emacs-version)
-	    (setq error-message "Not support Emacs 18/19 nor Mule 1\n")
+	   ((featurep 'xemacs)
+	    (setq error-message "Not support XEmacs\n")
+	    (error ""))
+	   ((string-match "^\\(18\\|19\\|20\\)" emacs-version)
+	    (setq error-message "Not support Emacs 18/19/20 nor Mule 1\n")
 	    (error "")))
 	  ;; initializing
 	  (or no-dir (mew-buffers-init))
 	  (or no-dir (mew-temp-dir-init))
 	  (mew-mark-init)
 	  (mew-config-init)
+	  (mew-subprocess-init)
 	  (mew-rotate-log-files mew-smtp-log-file)
 	  (mew-rotate-log-files mew-nntp-log-file)
 	  (mew-rotate-log-files mew-refile-log-file))
@@ -193,10 +204,9 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 ;;;
 
 (defun mew-folder-init (folders)
-  (let (folder dir)
-    (while folders
-      (setq folder (mew-canonicalize-folder (car folders)))
-      (setq folders (cdr folders))
+  (let (dir)
+    (dolist (folder folders)
+      (setq folder (mew-canonicalize-folder folder))
       (setq dir (mew-expand-folder folder))
       (unless (file-exists-p dir)
 	(mew-make-directory dir)
@@ -204,31 +214,36 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 	(message "%s was created" dir))
       (setq dir (file-chase-links dir))
       (if (file-exists-p dir)
-	  (if (/= mew-folder-mode (mew-file-get-mode dir))
+	  (if (/= mew-folder-mode (file-modes dir))
 	      (set-file-modes dir mew-folder-mode))))))
 
-(defun mew-status-update (arg)
-  "Read Addrbook and update its information. 
-If executed with '\\[universal-argument]', information about local folders is 
-also updated in addition to that of Addrbook. 
-If executed with '1', information about local folders is updated.
-If executed with '2', information about newsgroups is updated.
-If executed with '3', information about IMAP folders is updated."
+(defun mew-status-update (&optional arg)
+  "Read Addrbook and update its information.
+If executed with '\\[universal-argument]', information about folders
+of the current world is also updated."
   (interactive "P")
-  (message "Updating status...")
   (cond
-   ((eq arg 1)
-    (mew-local-update (interactive-p)))
-   ((eq arg 2)
-    (mew-nntp-update))
-   ((eq arg 3)
-    (mew-imap-update))
+   ((and arg (integerp arg))
+    (mew-message-for-summary "This command was obsoleted. Type '\\[universal-argument]\\[mew-status-update]' to collect folders"))
+   ((consp arg)
+    (let (case proto)
+      (mew-set '(case proto) (mew-summary-case-proto))
+      (cond
+       ((mew-folder-localp proto)
+	(mew-local-update (mew-called-interactively-p)))
+       ((mew-folder-popp proto)
+	(message "Nothing updated for POP"))
+       ((mew-folder-imapp proto)
+	(mew-imap-update case))
+       ((mew-folder-nntpp proto)
+	(mew-nntp-update case)))))
    (t
-    (if (interactive-p) (mew-set-environment 'no-dir))
+    (if (mew-called-interactively-p) (mew-set-environment 'no-dir))
     ;; These two must be before mew-local-update
+    (message "Updating status...")
     (mew-config-setup)
     (mew-regex-setup)
-    (if arg (mew-local-update (interactive-p))) ;; arg == '(4)
+    (if arg (mew-local-update (mew-called-interactively-p)))
     (mew-folder-init mew-basic-folders)
     (mew-folder-init mew-inbox-folders)
     (mew-folder-init mew-queue-folders)
@@ -244,29 +259,25 @@ If executed with '3', information about IMAP folders is updated."
     (mew-thread-setup)
     (mew-decoration-setup)
     (mew-biff-setup)
-    (mew-ct-setup)))
-  (mew-nmz-clean-up (interactive-p))
-  (unless (or (eq arg 2) (eq arg 3))
-    (message "Updating status...done")))
+    (mew-ct-setup)
+    (run-hooks 'mew-status-update-hook)
+    (message "Updating status...done"))))
 
 (defvar mew-mime-content-type-list nil
   "Candidate of Content-Type: when CT: is changed in draft buffer.")
 
 (defun mew-ct-setup ()
-  (let ((cts mew-mime-content-type) ct)
-    (while cts
-      (setq ct (car (car cts)))
-      (setq cts (cdr cts))
-      (if (and (stringp ct) (not (string-match "/$" ct)))
-	  (setq mew-mime-content-type-list
-		(cons (capitalize ct) mew-mime-content-type-list))))))
+  (dolist (ct (mapcar 'car mew-mime-content-type))
+    (if (and (stringp ct) (not (string-match "/$" ct)))
+	(setq mew-mime-content-type-list
+	      (cons (capitalize ct) mew-mime-content-type-list)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Current status
 ;;;
 
-(defsubst mew-frame-id (&optional frame)
+(defun mew-frame-id (&optional frame)
   (if window-system
       (let ((fram (or frame (selected-frame))))
 	(concat "mew-current-info-"
@@ -285,17 +296,14 @@ If executed with '3', information about IMAP folders is updated."
 
 (defun mew-current-clean-up ()
   (if window-system
-      (let ((frams (frame-list)))
-	(while frams
-	  (mew-info-clean-up
-	   (concat "mew-current-info-" (mew-frame-id (car frams))))
-	  (setq frams (cdr frams))))
+      (dolist (frame (frame-list))
+	(mew-info-clean-up (concat "mew-current-info-" (mew-frame-id frame))))
     (mew-info-clean-up "mew-current-info-no-window")))
 
-(defsubst mew-current-set-window-config ()
+(defun mew-current-set-window-config ()
   (mew-current-set-window (mew-frame-id) (current-window-configuration)))
 
-(defsubst mew-current-get-window-config ()
+(defun mew-current-get-window-config ()
   (let ((win (mew-current-get-window (mew-frame-id))))
     (unless (window-configuration-p win)
       (setq win mew-inbox-window))
@@ -343,11 +351,10 @@ If executed with '3', information about IMAP folders is updated."
 		     (string-match "^mew-" (symbol-name major-mode)))
 	   (setq wins (cons win wins))))
        nil)
-      (while wins
+      (dolist (win wins)
 	(unless (one-window-p)
 	  (setq ret t)
-	  (delete-window (car wins)))
-	(setq wins (cdr wins)))
+	  (delete-window win)))
       (select-window owin)
       ret)))
 
@@ -358,13 +365,13 @@ Predefined symbol is 'summary, 'message, and 'draft.
 They are used as a key of 'mew-window-configuration to get
 a list of two numbers.
 
-Two numbers means the ratio of the upper window and 
+Two numbers means the ratio of the upper window and
 the lower window (i.e. the window of Message buffer).
 
 If Message buffer does not exist, it will be created. If the height of
 the lower window is not zero, switch to the buffer."
   (let* ((windows
-	  (if (listp action) 
+	  (if (listp action)
 	      action
 	    (car (cdr (assq action mew-window-configuration)))))
 	 (msgbuf  (mew-buffer-message))
@@ -385,8 +392,8 @@ the lower window is not zero, switch to the buffer."
     ;;
     (if (get-buffer msgbuf)
 	(delete-windows-on msgbuf)
-      (save-excursion
-	(set-buffer (get-buffer-create msgbuf))
+      (with-current-buffer (get-buffer-create msgbuf)
+	(kill-all-local-variables)
 	;; "truncate?" is asked in Message mode.
 	;; so set the same toolbar as Summary mode
 	(mew-summary-toolbar-update)
@@ -396,7 +403,7 @@ the lower window is not zero, switch to the buffer."
     (unless (zerop (nth 0 windows))
       (setq sum-height (max window-min-height
 			    (/ (* height (nth 0 windows)) winsum))))
-    (if (and (eq action 'message) (= (% sum-height 2) 1)) 
+    (if (and (eq action 'message) (= (% sum-height 2) 1))
 	(setq sum-height (1+ sum-height)))
     (unless (zerop (nth 1 windows))
       (setq msg-height (max window-min-height (- height sum-height))))
@@ -414,14 +421,13 @@ the lower window is not zero, switch to the buffer."
 ;;;
 
 (defun mew-window-number ()
-  (let ((x (car (mew-window-edges (selected-window))))
+  (let ((x (car (window-edges (selected-window))))
 	(i 1))
     (walk-windows
      (lambda (win)
-       (when (save-excursion
-	       (set-buffer (window-buffer win))
+       (when (with-current-buffer (window-buffer win)
 	       (mew-summary-or-virtual-p))
-	 (if (< (car (mew-window-edges win)) x)
+	 (if (< (car (window-edges win)) x)
 	     (setq i (1+ i)))))
      'nominibuf)
     i))
@@ -433,11 +439,9 @@ the lower window is not zero, switch to the buffer."
 	 (n (mew-window-number))
 	 (i 0))
     (catch 'loop
-      (while frames
-	(if (equal me (car frames))
-	    (throw 'loop i))
-	(setq i (1+ i))
-	(setq frames (cdr frames))))
+      (dolist (frame frames)
+	(if (equal me frame) (throw 'loop i))
+	(setq i (1+ i))))
     (setq i (- len i 1))
     (if (= n 1)
 	(format "%s%d" mew-buffer-message i)
@@ -457,16 +461,12 @@ the lower window is not zero, switch to the buffer."
   (mew-addq mew-buffers folder))
 
 (defun mew-buffers-bury ()
-  (let ((buffers mew-buffers))
-    (while buffers
-      (if (get-buffer (car buffers))
-	  (bury-buffer (car buffers)))
-      (setq buffers (cdr buffers)))))
+  (dolist (buffer mew-buffers)
+    (if (get-buffer buffer) (bury-buffer buffer))))
 
 (defun mew-buffers-clean-up ()
-  (while mew-buffers
-    (mew-remove-buffer (car mew-buffers))
-    (setq mew-buffers (cdr mew-buffers)))
+  (dolist (buffer mew-buffers)
+    (mew-remove-buffer buffer))
   (mew-buffers-init))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -488,7 +488,7 @@ for privacy/speed reasons."
   (add-hook 'kill-emacs-hook 'mew-temp-dir-clean-up))
 
 (defun mew-temp-dir-clean-up ()
-  "A function to remove Mew's temporary directory recursively. 
+  "A function to remove Mew's temporary directory recursively.
 It is typically called by kill-emacs-hook."
   (remove-hook 'kill-emacs-hook 'mew-temp-dir-clean-up)
   (if (and mew-temp-dir (file-exists-p mew-temp-dir))
@@ -512,14 +512,14 @@ It is typically called by kill-emacs-hook."
 (defvar mew-regex-ignore-folders nil)
 (defvar mew-regex-thread-separator nil)
 
-(defsubst mew-mark-regex (mark)
+(defun mew-mark-regex (mark)
   (concat "^" (regexp-quote (char-to-string mark))))
 
-(defsubst mew-mark-list-regex (mark-list)
+(defun mew-mark-list-regex (mark-list)
   (concat "^" "[" (mapconcat 'char-to-string mark-list "") "]"))
 
 (defun mew-regex-setup-msg (lst)
-  (setq mew-regex-msg-show-list lst) 
+  (setq mew-regex-msg-show-list lst)
   (setq mew-regex-msg-show (mew-mark-list-regex lst))
   (setq mew-regex-msg-or-part (concat mew-regex-msg-show "\\|" mew-regex-part)))
 
@@ -571,13 +571,12 @@ It is typically called by kill-emacs-hook."
 
 (defun mew-decoration-setup ()
   (interactive)
-  (when mew-gemacs-p
-    (put-text-property 0 (length mew-end-of-message-string)
-		       'face 'mew-face-eof-message
-		       mew-end-of-message-string)
-    (put-text-property 0 (length mew-end-of-part-string)
-		       'face 'mew-face-eof-part
-		       mew-end-of-part-string)))
+  (put-text-property 0 (length mew-end-of-message-string)
+		     'face 'mew-face-eof-message
+		     mew-end-of-message-string)
+  (put-text-property 0 (length mew-end-of-part-string)
+		     'face 'mew-face-eof-part
+		     mew-end-of-part-string))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -595,8 +594,7 @@ It is typically called by kill-emacs-hook."
 	 (folder (if (bufferp buf) (buffer-name buf) buf))
 	 obuf)
     (if (get-buffer buf)
-	(save-excursion
-	  (set-buffer buf)
+	(with-current-buffer buf
 	  (when (mew-summary-or-virtual-p)
 	    (mew-summary-kill-subprocess)
 	    (setq obuf (mew-local-buffer-name folder))
@@ -607,18 +605,16 @@ It is typically called by kill-emacs-hook."
 
 (defun mew-buffer-clean-up (regex &optional func)
   (unless func (setq func 'mew-kill-buffer))
-  (let ((bufs (mew-buffer-list regex)))
-    (while bufs
-      (funcall func (car bufs))
-      (setq bufs (cdr bufs)))))
+  (dolist (buf (mew-buffer-list regex))
+    (funcall func buf)))
 
-(defsubst mew-quit-toolbar-update ()
+(defun mew-quit-toolbar-update ()
   (mew-redraw) ;; due to mouse-face bug
   (if (fboundp 'redraw-frame) ;; for BOW
       (redraw-frame (selected-frame)))) ;; update toolbar
 
 (defun mew-summary-suspend ()
-  "Suspend Mew then switch to another buffer. All buffers of 
+  "Suspend Mew then switch to another buffer. All buffers of
 Mew remain, so you can resume with buffer operations."
   (interactive)
   (mew-buffer-clean-up
@@ -644,6 +640,7 @@ Mew remain, so you can resume with buffer operations."
     ;;
     (mew-passwd-clean-up) ;; should be before dir clean up
     (mew-temp-dir-clean-up)
+    (mew-subprocess-clean-up)
     ;;
     (run-hooks 'mew-quit-hook)
     ;;
@@ -656,7 +653,6 @@ Mew remain, so you can resume with buffer operations."
     (mew-highlight-timer-clean-up)
     (mew-net-clean-up)
     (mew-biff-clean-up)
-    (mew-nmz-clean-up)
     ;;
     (mew-window-pop)
     (mew-window-clean-up)
@@ -667,7 +663,7 @@ Mew remain, so you can resume with buffer operations."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Transit
+;;; Transit (should be removed someday)
 ;;;
 
 ;; Spotlight does not handle Files/Directories starting with ".".
@@ -675,21 +671,16 @@ Mew remain, so you can resume with buffer operations."
   '(("+.pop" "+#pop") ("+.imap" "+#imap") ("+.nntp" "+#nntp")))
 
 (defun mew-transit-folder ()
-  (let ((alist mew-transit-folder-alist)
-	ent old new dirs subold subnew)
+  (let (old new dirs subnew)
     (with-temp-buffer
-      (while alist
-	(setq ent (car alist))
-	(setq alist (cdr alist))
+      (dolist (ent mew-transit-folder-alist)
 	(setq old (mew-expand-folder (nth 0 ent)))
 	(setq new (mew-expand-folder (nth 1 ent)))
 	(when (and (file-directory-p old) (not (file-exists-p new)))
 	  (rename-file old new)
 	  (cd new)
 	  (setq dirs (directory-files "."))
-	  (while dirs
-	    (setq subold (car dirs))
-	    (setq dirs (cdr dirs))
+	  (dolist (subold dirs)
 	    ;; % is a special character for URL.
 	    ;; And "open" does not convert % to %25, sigh.
 	    (when (string-match "@[^%]+\\(%\\)" subold)
@@ -748,17 +739,20 @@ Mew remain, so you can resume with buffer operations."
 (require 'mew-imap2)
 (require 'mew-config)
 (require 'mew-auth)
-(require 'mew-nmz)
+
+(eval-when-compile
+  (if (mew-which-el "timer")
+      (require 'timer)))
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2007 Mew developing team.
+;; Copyright (C) 1994-2023 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -767,7 +761,7 @@ Mew remain, so you can resume with buffer operations."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
